/**
 * @Description: 
 * @version: 
 * @Author: 北京四维空间数码科技有限公司-研发部
 * @Date: 2023-01-11 13:52:38
 * @LastEditors: 北京四维空间数码科技有限公司-研发部
 * @LastEditTime: 2023-03-03 14:12:11
 */
import axios from 'axios';
import NProgress from 'nprogress'
import { ElMessage, ElLoading} from "element-plus";
import { tansParams, blobValidate } from "@/utils/common.js";
import { saveAs } from 'file-saver'
let downloadLoadingInstance;
axios.defaults.headers["Content-Type"] = "application/json;charset=utf-8";

const service = axios.create({
    baseURL: import.meta.env.VITE_BASE_API,
    // withCredentials: true,
    timeout: 100000
})

service.interceptors.request.use((config) => {
    NProgress.start()
    return config
}, (error) => {
    NProgress.done()
    console.log(error);
    ElMessage.error(error)
    return Promise.reject(error);
})

service.interceptors.response.use((response) => {
    NProgress.done()
    const res = response.data
    // 未设置状态码则默认成功状态
    res.code = res.code || 200;
    if (res.code !== '200' && res.code !== 200) {
        console.log(res.message || res.msg || 'Error');
        ElMessage({
            message: res.message || res.msg || 'Error',
            type: 'warning',
        })
        return Promise.reject(new Error(res.message || res.msg || 'Error'))
    } else {
        return res
    }
}, (error) => {
    NProgress.done()
    console.log('err' + error)
    ElMessage.error('请求失败，请稍后再试')
    return Promise.reject(error)
})
// 通用下载方法
export function download(url, params, filename,noloading) {
  if(!noloading){
    downloadLoadingInstance = ElLoading.service({
        text: "正在下载数据，请稍候",
        background: "rgba(0, 0, 0, 0.7)",
    });
  }
  return service.post(url, params, {
      transformRequest: [(params) => { return tansParams(params) }],
      headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
      responseType: 'blob'
    }).then(async (data) => {
      const isLogin = await blobValidate(data);
      if (isLogin) {
        const blob = new Blob([data])
        saveAs(blob, filename)
      } else {
        const resText = await data.text();
        const rspObj = JSON.parse(resText);
        const errMsg = errorCode[rspObj.code] || rspObj.msg || errorCode['default']
        ElMessage.error(errMsg);
      }
      downloadLoadingInstance && downloadLoadingInstance.close();
    }).catch((r) => {
      console.error(r)
      ElMessage.error('下载文件出现错误，请联系管理员！')
      downloadLoadingInstance && downloadLoadingInstance.close();
    })
  }
export default service;