import * as echarts from 'echarts'
const CubeLeft = echarts.graphic.extendShape({
    shape: {
        x: 0,
        y: 0
    },
    buildPath: function (ctx, shape) {
        const xAxisPoint = shape.xAxisPoint
        const c0 = [shape.x, shape.y]
        const c1 = [shape.x - 9, shape.y - 9]
        const c2 = [xAxisPoint[0] - 9, xAxisPoint[1] - 9]
        const c3 = [xAxisPoint[0], xAxisPoint[1]]
        ctx.moveTo(c0[0], c0[1]).lineTo(c1[0], c1[1]).lineTo(c2[0], c2[1]).lineTo(c3[0], c3[1]).closePath()
    }
})
const CubeRight = echarts.graphic.extendShape({
    shape: {
        x: 0,
        y: 0
    },
    buildPath: function (ctx, shape) {
        const xAxisPoint = shape.xAxisPoint
        const c1 = [shape.x, shape.y]
        const c2 = [xAxisPoint[0], xAxisPoint[1]]
        const c3 = [xAxisPoint[0] + 18, xAxisPoint[1] - 9]
        const c4 = [shape.x + 18, shape.y - 9]
        ctx.moveTo(c1[0], c1[1]).lineTo(c2[0], c2[1]).lineTo(c3[0], c3[1]).lineTo(c4[0], c4[1]).closePath()
    }
})
const CubeTop = echarts.graphic.extendShape({
    shape: {
        x: 0,
        y: 0
    },
    buildPath: function (ctx, shape) {
        const c1 = [shape.x, shape.y]
        const c2 = [shape.x + 18, shape.y - 9]
        const c3 = [shape.x + 9, shape.y - 18]
        const c4 = [shape.x - 9, shape.y - 9]
        ctx.moveTo(c1[0], c1[1]).lineTo(c2[0], c2[1]).lineTo(c3[0], c3[1]).lineTo(c4[0], c4[1]).closePath()
    }
})
echarts.graphic.registerShape('CubeLeft', CubeLeft)
echarts.graphic.registerShape('CubeRight', CubeRight)
echarts.graphic.registerShape('CubeTop', CubeTop)

export const barOption = (data, details) => {
    var option = {
        tooltip: {
            trigger: "axis",
            formatter: function (params) {
                var str = '';
                str += '<span style=\"display:inline-block;margin-right:4px;border-radius:10px;width:10px;height:10px;background-color:#9ec1dc;\"></span>' + params[0].seriesName + '<br/>';
                str += params[0].value + '（万亩）<br/>';
                str += '<span style=\"display:inline-block;margin-right:4px;border-radius:10px;width:10px;height:10px;background-color:#57a2d8;\"></span>' + params[1].seriesName + '<br/>';
                str += params[1].value + '（万亩）<br/>';
                return str;
            }
        },
        grid: {
            left: 10,
            right: 10,
            bottom: '2%',
            top: 0,
            containLabel: true
        },
        xAxis: {
            type: 'category',
            data: [data.name],
            axisLine: {
                show: false,
                lineStyle: {
                    color: 'white'
                }
            },
            axisLabel: {
                show: false,
                fontSize: 10
            }
        },
        yAxis: {
            axisLine: {
                show: false,
            },
            splitLine: {
                show: false
            },
            axisTick: {
                show: false
            },
            axisLabel: {
                show: false
            },
        },
        series: [{
            name: details.name[0],
            type: 'custom',
            renderItem: function (params, api) {
                const location = api.coord([api.value(0), api.value(1)])
                return {
                    type: 'group',
                    children: [{
                        type: 'CubeLeft',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: 'rgba(47,102,192,.27)'
                        }
                    }, {
                        type: 'CubeRight',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: 'rgba(59,128,226,.27)'
                        }
                    }, {
                        type: 'CubeTop',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: 'rgba(72,156,221,.27)'
                        }
                    }]
                }
            },
            data: [data[details.key[0]]]
        }, {
            name: details.name[1],
            type: 'custom',
            renderItem: (params, api) => {
                const location = api.coord([api.value(0), api.value(1)])
                return {
                    type: 'group',
                    children: [{
                        type: 'CubeLeft',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: new echarts.graphic.LinearGradient(0, 0, 0, 1, [{
                                offset: 0,
                                color: '#3B80E2'
                            },
                            {
                                offset: 1,
                                color: '#49BEE5'
                            }
                            ])
                        }
                    }, {
                        type: 'CubeRight',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: new echarts.graphic.LinearGradient(0, 0, 0, 1, [{
                                offset: 0,
                                color: '#3B80E2'
                            },
                            {
                                offset: 1,
                                color: '#49BEE5'
                            }
                            ])
                        }
                    }, {
                        type: 'CubeTop',
                        shape: {
                            api,
                            xValue: api.value(0),
                            yValue: api.value(1),
                            x: location[0],
                            y: location[1],
                            xAxisPoint: api.coord([api.value(0), 0])
                        },
                        style: {
                            fill: new echarts.graphic.LinearGradient(0, 0, 0, 1, [{
                                offset: 0,
                                color: '#3B80E2'
                            },
                            {
                                offset: 1,
                                color: '#49BEE5'
                            }
                            ])
                        }
                    }]
                }
            },
            data: [data[details.key[1]]]
        }, {
            type: 'bar',
            label: {
                normal: {
                    show: true,
                    position: 'bottom',
                    fontSize: 13,
                    // color: '#fff',
                    offset: [4, -75],
                    formatter: (params) => {
                        return data[details.key[1]] + '/' + data[details.key[0]]
                    },
                }
            },
            itemStyle: {
                color: 'transparent'
            },
            data: [data[details.key[0]]]
        }]
    }
    return option;
}

export const locationXy = (id) => {
    switch (id) {
        case "150502": return [13615264.947860649, 5415246.494936824];
        case "150581": return [13308801.721789824, 5702679.422572563];
        case "150523": return [13508397.568782162, 5441363.345619797];
        case "150522": return [13668920.274506062, 5351049.759446745];
        case "150521": return [13638530.053519499, 5481891.613494568];
        case "150524": return [13532442.578793507, 5289957.717064263];
        case "150525": return [13453851.018293457, 5316417.441335446];
        case "150526": return [13423126.838834513, 5606923.837556483];
    }
}

export const formatKeyValue = (type) => {
    switch (type) {
        case "NcTdccjsgm":
            var deatils = {
                name: ['可承载耕地面积', '现状耕地面积'],
                key: ['kczgdmj2', 'xzgdmj2'],
            }
            return deatils;

        case "CzTdccjsgm":
            var deatils = {
                name: ['可承载建设规模', '现状城镇建设用地面积'],
                key: ['kczjsgm', 'xzczjsydmj'],
            }
            return deatils;
        case "Nyszy":
            var deatils = {
                name: ['可承载耕地规模', '现状耕地面积'],
                key: ['kczdgdgm2', 'xzgdmj2'],
            }
            return deatils;
        case "Czszy":
            var deatils = {
                name: ['可承载城镇建设用地规模', '现状城镇建设用地规模'],
                key: ['kczczjsydgm', 'xzczjsydmj'],
            }
            return deatils;
    }
}