import "ol/ol.css";
import { Map, View } from "ol";
import TileLayer from "ol/layer/Tile";
import Layer from 'ol/layer/Layer';
import XYZ from "ol/source/XYZ";
import { fromLonLat, get, transform } from "ol/proj";
import MousePosition from "ol/control/MousePosition";
import VectorLayer from "ol/layer/Vector";
import VectorSource from "ol/source/Vector";
import ImageLayer from "ol/layer/Image";
import ImageWMS from "ol/source/ImageWMS";
import { Fill, Stroke, Circle, Style } from "ol/style";
import Feature from "ol/Feature";
import LinearRing from "ol/geom/LinearRing";
import TileWMS from "ol/source/TileWMS";
import WFS from "ol/format/WFS";
import GeoJSON from "ol/format/GeoJSON";
import WKT from "ol/format/WKT";
import { intersects, like } from "ol/format/filter";
import MapConfig from "@/utils/mapconfig";
import Overlay from "ol/Overlay";
import arcMap from "@/utils/arcMap.js";
import { buffer, getCenter } from "ol/extent";
import {
    LineString,
    MultiLineString,
    MultiPoint,
    MultiPolygon,
    Point,
    Polygon,
} from "ol/geom";
// import _FormItem from "element-plus/lib/el-form-item";
import request from "@/utils/requestTwo.js";
import * as echarts from "echarts";
import {
    barOption,
    locationXy,
    formatKeyValue,
} from "@/utils/echartsoption.js";
import { Base64 } from "js-base64";
// let Base64 = require('js-base64').Base64;
var map;
export const baseLayer = {
    /*天地图影像*/
    tdtImg() {
        if (!BaseMap.tdtImg) return null;
        return new TileLayer({
            //天地图
            id: "tdtImg",
            name: "tdtImg",
            zIndex: -1000000000,
            visible: true,
            source: new XYZ({
                crossOrigin: "anonymous",
                url: BaseMap.tdtImg,
            }),
        });
    },
    /*天地图注记影像*/
    tdtImgAnn() {
        if (!BaseMap.tdtImgAnn) return null;
        return new TileLayer({
            //天地图
            id: "tdtImgAnn",
            zIndex: -999999999,
            name: "tdtImgAnn",
            visible: true,
            source: new XYZ({
                crossOrigin: "anonymous",
                url: BaseMap.tdtImgAnn,
            }),
        });
    },
    /*天地图矢量*/
    tdtVec() {
        if (!BaseMap.tdtVec) return null;
        return new TileLayer({
            //天地图
            id: "tdtVec",
            name: "tdtVec",
            zIndex: -1000000000,
            visible: false,
            source: new XYZ({
                crossOrigin: "anonymous",
                url: BaseMap.tdtVec,
            }),
        });
    },
    /*天地图注记矢量*/
    tdtVecAnn() {
        if (!BaseMap.tdtVecAnn) return null;
        return new TileLayer({
            //天地图
            id: "tdtVecAnn",
            zIndex: -999999999,
            name: "tdtVecAnn",
            visible: false,
            source: new XYZ({
                crossOrigin: "anonymous",
                url: BaseMap.tdtVecAnn,
            }),
        });
    },
    /*界线*/
    jxMap() {
        if (!BaseMap.jxMap) return null;
        return new TileLayer({
            //界线
            id: "jxMap",
            zIndex: -999999999,
            name: "jxMap",
            visible: true,
            source: new XYZ({
                crossOrigin: "anonymous",
                url: BaseMap.jxMap,
            }),
        });
    },
    cityLine() {
        return new VectorLayer({
            name: "cityLine",
            source: new VectorSource({
                features: [],
            }),
            style: new Style({
                stroke: new Stroke({
                    color: "#f02fff",
                    width: 2,
                    lineDash: [1, 2, 3, 4, 5, 6],
                }),
            }),
            zIndex: 9,
            visible: false,
        });
    },
};

const myMap = (function() {
    var myMaps = {
        _basemap: null,
        _domid: null,
        _EPSG: null,
        EPSG: {
            C3857: "EPSG:3857",
            C4326: "EPSG:4326",
            C4490: "EPSG:4490", //国家2000
            C4610: "EPSG:4610", //西安1980
        },
        //地图
        map: {},
        //对比的地图
        map1: null,
        map2: null,
        map3: null,
        map4: null,
        //比较地图
        compareMap(type, view, layer) {
            var format = "image/png";
            var imgsource = new TileWMS({
                ratio: 1,
                url: BaseMap.geoImg,
                params: {
                    FORMAT: format,
                    VERSION: "1.1.1",
                    tilad: true,
                    STYLES: "",
                    LAYERS: layer,
                },
            });
            arcMap[type] = myMap[type] = new Map({
                layers: [
                    new TileLayer({
                        name: layer,
                        source: imgsource,
                    }),
                ],
                view: view,
                target: type,
            });
            var la = new VectorLayer({
                name: "compare-feature",
                declutter: false,
                source: new VectorSource({
                    features: [],
                }),
                style: new Style({
                    stroke: new Stroke({
                        color: "#1171d6",
                        width: 4,
                        lineDash: [1, 2, 3, 4, 5, 6],
                    }),
                }),
                zIndex: 9,
                visible: true,
            });
            myMap[type].addLayer(la, type);
            var _this = this;
            myMap[type].on("singleclick", function(evt) {
                var lonlat = myMap.webMercatorToWgs84(
                    evt.coordinate[0],
                    evt.coordinate[1]
                );
                var view = myMap[type].getView();
                myMap.SearchWfsByLoc(
                    layer,
                    `${lonlat[0]},${lonlat[1]}`,
                    view.getResolution() * 6.7,
                    function(d, f) {
                        if (d.length) {
                            var layerConfig = MapConfig[layer];
                            var layers = myMap.getLayerByNameAndMap(layer, type);
                            layers.popupInfo = layerConfig.popupInfo;
                            for (var i = 1; i <= 4; i++) {
                                if (myMap["map" + i]) {
                                    myMap["map" + i].removeOverlay(
                                        _this["map" + i].getOverlayById("popup_temp" + "map" + i)
                                    );
                                    var _cmap = myMap.getLayerByNameAndMap(
                                        "compare-feature",
                                        "map" + i
                                    );
                                    _cmap.getSource().clear();
                                    _cmap.getSource().addFeatures(d);
                                }
                            }
                            layers.popupInfo(d[0], type, evt);
                            myMap[type].updateSize();
                        }
                    }
                );
            });
        },
        loadMap: function(domId, par = {}) {
            var _op = {
                center: MapCenter.point,
                zoom: 8,
                base: [
                    "tdtImg",
                    "tdtImgAnn",
                    "tdtVec",
                    "tdtVecAnn",
                    "jxMap",
                    "cityLine",
                ],
                view: null,
            };
            var option = Object.assign({}, _op, par);
            //构建默认地图视图
            if (!option.view) {
                option.view = new View({
                    projection: get("EPSG:3857"),
                    center: fromLonLat(option.center),
                    zoom: option.zoom,
                    units: "m",
                    minZoom: 2,
                });
            }

            var _map = option.map || "map";
            myMap._EPSG = myMap.EPSG.C3857;
            this._domid = domId;
            this._basemap = [];
            for (var i = 0; i < option.base.length; i++) {
                var _layer = baseLayer[option.base[i]]();
                if (_layer) {
                    if (i == 0) {
                        _layer.setVisible(true);
                    }
                    myMap._basemap.push(_layer);
                }
            }
            myMap[_map] = arcMap[_map] = new Map({
                target: domId,
                layers: this._basemap,
                view: new View({
                    projection: get("EPSG:3857"),
                    center: fromLonLat(option.center),
                    zoom: option.zoom,
                    units: "m",
                    minZoom: 2,
                }),
            });
            this.tempLayers.init();
            if (option.click) {
                myMap.map.on("singleclick", this.mapOnClick);
            }
            if (!option.hideCity) this.cityLineDefault(!par.zoom);
            else if (!par.zoom) myMap.zoomToextent(MapCenter.extent);
        },
        mapOnClick: function(evt) {
            var pixel = myMap.map.getEventPixel(evt.originalEvent);
            var hit = myMap.map.hasFeatureAtPixel(pixel);
            if (hit) {
                myMap.map.forEachFeatureAtPixel(pixel, function(feature, layer) {
                    if (!layer) {
                        return;
                    }
                    //显示弹窗
                    function popShow(config, html) {
                        arcMap.popupInfo({
                                feature: feature,
                                position: evt,
                                title: feature.get(config.title),
                                attrs: config.attrs,
                            },
                            html
                        );
                        arcMap.map.updateSize();
                    }
                    //如果是方法就回调，否则检查配置
                    if (typeof layer.popupInfo == "function") {
                        layer.popupInfo(feature, popShow, evt);
                    } else {
                        var layerConfig = MapConfig[layer.get("name")];
                        if (layer.popupInfo && layerConfig && layerConfig.popupInfo) {
                            popShow(layerConfig);
                        }
                    }
                });
            } else {
                myMap.map.removeOverlay(myMap.getPopupOverlay());
            }
        },
        tempLayers: {
            _draw: "dview_draw_Layer",
            _search: "dview_search_Layer",
            _fliter: "dview_fliter_Layer",
            _factor: "dview_factor_Layer",
            _location: "dview_location_Layer",
            _redlayer: "redtemp_Layer",
            _center: [109.77, 39.57],
            dview_draw_Layer: null,
            dview_search_Layer: null,
            dview_fliter_Layer: null,
            dview_factor_Layer: null,
            dview_location_Layer: null,
            redtemp_Layer: null,
            init: function() {
                if (!myMap.getLayer(this._draw)) {
                    myMap.addLayer(myMap.CreateVecLayer(this._draw));
                    this.dview_draw_Layer = myMap.getLayer(this._draw);
                }
                if (!myMap.getLayer(this._search)) {
                    myMap.addLayer(myMap.CreateVecLayerOfPopup(this._search));
                    this.dview_search_Layer = myMap.getLayer(this._search);
                    this.dview_search_Layer.setStyle(myMap.SetSelectStyle);
                }
                if (!myMap.getLayer(this._location)) {
                    myMap.addLayer(
                        myMap.CreateVecLayer(
                            this._location,
                            new Style({
                                fill: new Fill({
                                    color: "rgba(255, 255, 255, 0.05)",
                                }),
                                stroke: new Stroke({
                                    color: "#FF0000",
                                    width: 2,
                                }),
                                image: new Circle({
                                    radius: 7,
                                    fill: new Fill({
                                        color: "#FF0000",
                                    }),
                                }),
                            }),
                            null,
                            99999
                        )
                    );
                    this.dview_location_Layer = myMap.getLayer(this._location);
                }
                if (!myMap.getLayer(this._fliter)) {
                    var style = new Style({
                        fill: new Fill({
                            color: "rgba(0, 120, 215,0.01)",
                        }),
                        stroke: new Stroke({
                            color: "#0078d7",
                            width: 2,
                        }),
                    });
                    myMap.addLayer(myMap.CreateVecLayer(this._fliter, style));
                    this.dview_fliter_Layer = myMap.getLayer(this._fliter);
                }
                if (!myMap.getLayer(this._factor)) {
                    var style = new Style({
                        image: new Circle({
                            radius: 7,
                            fill: new Fill({
                                color: "#1171d6",
                            }),
                        }),
                        stroke: new Stroke({
                            color: "#1171d6",
                            width: 2,
                            lineDash: [1, 2, 3, 4, 5, 6],
                        }),
                    });
                    myMap.addLayer(myMap.CreateVecLayer(this._factor, style));
                    this.dview_factor_Layer = myMap.getLayer(this._factor);
                }
                //规划用地用海
                if (!myMap.getLayer(this._redlayer)) {
                    var style = new Style({
                        image: new Circle({
                            radius: 7,
                            fill: new Fill({
                                color: "#FF0000",
                            }),
                        }),
                        stroke: new Stroke({
                            color: "#FF0000",
                            width: 4,
                            lineDash: [1, 2, 3, 4, 5, 6],
                        }),
                    });
                    myMap.addLayer(myMap.CreateVecLayer(this._redlayer, style));
                    this.redtemp_Layer = myMap.getLayer(this._redlayer);
                }
            },
            clear: function(layer) {
                if (layer) {
                    layer.getSource().clear();
                } else {
                    this.dview_draw_Layer.getSource().clear();
                    this.dview_search_Layer.getSource().clear();
                    this.dview_fliter_Layer.getSource().clear();
                    this.dview_factor_Layer.getSource().clear();
                    this.dview_location_Layer.getSource().clear();
                    this.redtemp_Layer.getSource().clear();
                }
            },
        },
        cityLineDefault(zoom) {
            request({
                url: `/js/${XZQ_DM}.geojson`,
                method: "get",
            }).then((res) => {
                var f = new GeoJSON().readFeatures(Base64.decode(res));
                var city = baseLayer.cityLine();
                city.getSource().addFeatures(f);
                city.setVisible(true);
                myMap.addLayer(city);
                if (zoom) myMap.zoomToextent(f[0].getGeometry().getExtent());
            });
        },
        updateSize: function(settime) {
            var count = 0;

            function usize() {
                myMap.map.updateSize();
                for (var i = 1; i <= 4; i++) {
                    if (myMap["map" + i]) {
                        myMap["map" + i].updateSize();
                    }
                }
                if (count < 600) setTimeout(usize, 1);
            }
            if (settime) {
                setTimeout(usize, 1);
            } else usize();
        },
        //按name获取
        getLayer: function(name) {
            var layer_ = null;
            this.map.getLayers().forEach(function(layer, i) {
                if (
                    layer &&
                    layer.get("name") &&
                    layer.get("name") == name.toString()
                ) {
                    layer_ = layer;
                    return;
                }
            });
            return layer_;
        },
        //按id获取
        getLayerById: function(id, type) {
            var map = this.map;
            if (type) map = this[type];
            var layer_ = null;
            map.getLayers().forEach(function(layer, i) {
                if (layer && layer.get("id") && layer.get("id") == id) {
                    layer_ = layer;
                    return;
                }
            });
            return layer_;
        },
        //获取指定地图的图层
        getLayerByNameAndMap: function(name, type) {
            var layer_ = null;
            this[type].getLayers().forEach(function(layer) {
                if (layer && layer.get("name") && layer.get("name") == name) {
                    layer_ = layer;
                    return;
                }
            });
            return layer_;
        },
        zoomToextent: function(extent, meters) {
            meters = meters || 1000;
            var view = myMap.map.getView();
            view.fit(buffer(extent, meters), arcMap.map.getSize());
        },
        zoomToFeature: function(feature, meters) {
            meters = meters || 1000;
            let extent;
            //添加支持多个feature sky220622
            if (Array.isArray(feature)) {
                let cords = [];
                for (let i = 0; i < feature.length; i++) {
                    let cord = feature[i].getGeometry().getExtent();
                    cords.push([cord[0], cord[1]]);
                    cords.push([cord[2], cord[3]]);
                }
                let testfs = new Feature({
                    geometry: new LineString(cords),
                });
                extent = testfs.getGeometry().getExtent();
            } else {
                extent = feature.getGeometry().getExtent();
            }

            myMap.zoomToextent(extent, meters);
        },
        //缩放到中心点
        zoomToCenter: function(zoom) {
            var view = arcMap.map.getView();
            view.setZoom(zoom || 12);
            view.setCenter(fromLonLat(myMaps._center));
        },
        //缩放到图层 sky20220623
        zoomToLayer: function(layurl, cl) {
            var url = layurl + "?f=pjson"; //空间查询url;
            request({
                method: "get",
                url: url,
            }).then((res) => {
                var extent = [
                    res.fullExtent.xmin,
                    res.fullExtent.ymin,
                    res.fullExtent.xmax,
                    res.fullExtent.ymax,
                ];
                myMaps.zoomToextent(extent, 20000);
                if (cl) cl(extent);
            });
        },
        //移动
        moveToFeature: function(feature) {
            let extent;
            //sky220622 添加支持多个feature
            if (Array.isArray(feature)) {
                let cords = [];
                for (let i = 0; i < feature.length; i++) {
                    let cord = feature[i].getGeometry().getExtent();
                    cords.push([cord[0], cord[1]]);
                    cords.push([cord[2], cord[3]]);
                }
                let testfs = new Feature({
                    geometry: new LineString(cords),
                });
                extent = testfs.getGeometry().getExtent();
            } else {
                extent = feature.getGeometry().getExtent();
            }
            var center = getCenter(extent);
            var view = arcMap.map.getView();
            view.setCenter(center);
            arcMap.map.render();
        },
        addLayer: function(layer, type) {
            if (type) {
                this[type].addLayer(layer);
            } else {
                this.map.addLayer(layer);
            }
        },
        //删除图层
        removeLayer: function(layer) {
            var _layer = layer;
            if (typeof layer == "string") _layer = myMap.getLayer(layer);
            this.map.removeLayer(_layer);
        },
        //删除以XX开头的图层(图层名称)
        removeLayers: function(name) {
            var lys = this.map.getLayers().array_;
            for (var i = 0; i < lys.length; i++) {
                var layer = lys[i];
                if (
                    (layer &&
                        layer.get("name") &&
                        layer.get("name").indexOf(name) == 0) ||
                    (layer && layer.get("id") && layer.get("id").indexOf(name) == 0)
                ) {
                    this.map.removeLayer(layer);
                    i--;
                }
            }
        },
        CreateVecLayer: function(layername, style, popup, zindex) {
            style =
                style ||
                new Style({
                    fill: new Fill({
                        color: "rgba(255, 255, 255, 0.2)",
                    }),
                    stroke: new Stroke({
                        color: "#ffcc33",
                        width: 2,
                    }),
                    image: new Circle({
                        radius: 7,
                        fill: new Fill({
                            color: "#ffcc33",
                        }),
                    }),
                });
            var _VectorLayer = new VectorLayer({
                zIndex: zindex || 999,
                declutter: false,
                source: new VectorSource(),
                name: layername,
                style: style,
                visible: true,
            });
            if (popup) this.CreateLayerPopup(_VectorLayer, popup);
            return _VectorLayer;
        },
        SetSelectStyle: function(feature, resolution) {
            var strs = new Array();
            var l = feature.getId();
            if (l != undefined) {
                strs = feature.getId().split(".")[0];
                for (var val in MapConfig) {
                    var strL = (MapConfig[val].name || "").split(":")[1];
                    if (strs == strL) {
                        return MapConfig[val].style(feature);
                    }
                }
            }
            return new Style({
                fill: new Fill({
                    color: "rgba(255, 255, 255, 0.2)",
                }),
                stroke: new Stroke({
                    color: "#ffcc33",
                    width: 2,
                }),
                image: new Circle({
                    radius: 7,
                    fill: new Fill({
                        color: "#ffcc33",
                    }),
                }),
            });
        },
        CreateVecLayerOfPopup: function(layername, style) {
            return this.CreateVecLayer(layername, style, true);
        },
        CreateLayerPopup: function(layer, popup) {
            if (layer && typeof layer != "object") layer = myMap.getLayer(layer);
            if (layer) layer.popupInfo = popup;
            return layer;
        },
        lonLatToWebMercator: function(lon, lat) {
            var x = (lon * 20037508.34) / 180;
            var y =
                Math.log(Math.tan(((90 + lat) * Math.PI) / 360)) / (Math.PI / 180);
            y = (y * 20037508.34) / 180;
            return [x, y];
        },
        webMercatorToWgs84: function(x, y) {
            var lon = (x / 20037508.34) * 180;
            var lat = (y / 20037508.34) * 180;
            lat =
                (180 / Math.PI) *
                (2 * Math.atan(Math.exp((lat * Math.PI) / 180)) - Math.PI / 2);
            return [lon, lat];
        },
        featureToGeojson(f) {
            var geoJson = new GeoJSON().writeGeometry(f.getGeometry(), {
                dataProjection: get(myMap.EPSG.C4326),
                featureProjection: get(myMap._EPSG),
            });
            return geoJson;
        },
        geojsonFeatures(geojson) {
            var features = new GeoJSON().readFeatures(geojson, {
                dataProjection: get(myMap.EPSG.C4326),
                featureProjection: get(myMap._EPSG),
            });
            return features;
        },
        featureToWkt(f) {
            var geoJson = new WKT().writeGeometry(f.getGeometry(), {
                dataProjection: get(myMap.EPSG.C4326),
                featureProjection: get(myMap._EPSG),
            });
            return geoJson;
        },
        LonLatDisplay: function() {
            var mousePositionControl = new MousePosition({
                coordinateFormat: function(e) {
                    var es = myMap.webMercatorToWgs84(e[0], e[1]);
                    return (
                        "经度：" + es[0].toFixed(2) + "° , 纬度：" + es[1].toFixed(2) + "°"
                    );
                },
                projection: get("EPSG:3857"),
                className: "lonlat",
                target: document.getElementById("lonlat"),
                undefinedHTML: "&nbsp;",
            });
            mousePositionControl.setMap(myMap.map);
        },
        removePopupOverlay: function() {
            myMap.map.removeOverlay(myMap.getPopupOverlay());
        },
        getPopupOverlay: function() {
            return this.map.getOverlayById("popup_temp");
        },
        addOverlay: function(Overlay, type) {
            if (type) {
                this[type].addOverlay(Overlay);
            } else {
                this.map.addOverlay(Overlay);
            }
        },
        //图表overlayer
        addOverLayerByEcharts: function(data, type) {
            var details = formatKeyValue(type);
            var dom = document.querySelectorAll(".echartsOverlay");
            if (dom && dom.length) {
                dom.forEach((item) => {
                    item.remove();
                });
            }
            data.map((item) => {
                var code = item.code;
                if (type == "Nyszy" || type == "Czszy") {
                    code = item.xzqdm;
                }
                var ele =
                    "<div id='" + code + "' style='width:100px;height:100px;'></div>";
                var para = document.createElement("div");
                para.innerHTML = ele;
                var _div = document.getElementById(myMap._domid);
                _div.appendChild(para);
                var echartbar = {
                    chart: null,
                    option: barOption(item, details),
                };
                echartbar.chart = echarts.init(document.getElementById(code));
                echartbar.chart.setOption(echartbar.option, true);
                window.addEventListener("resize", function() {
                    echartbar.chart.resize();
                });
                var _overlay = new Overlay({
                    id: code,
                    position: locationXy(code),
                    positioning: "bottom-center",
                    element: document.getElementById(code),
                    stopEvent: true,
                    autoPan: true,
                    autoPanAnimation: {
                        duration: 250,
                    },
                    className: "echartsOverlay",
                });
                myMap.addOverlay(_overlay, "map");
            });
        },
        //地图弹窗--点
        popupInfo: function(option) {
            var feature = option.feature;
            var evt = option.position;
            var _popupId_ = "popup_temp" + option.index;
            if (typeof option.type == "string") {
                myMap[option.type].removeOverlay(myMap.map1.getOverlayById(_popupId_));
            } else {
                option.type = null;
                myMap.map1.removeOverlay(myMap.getPopupOverlay());
            }
            var html;
            if (!html) {
                html = "<div id='" + _popupId_ + "' class='map-popupinfo'>";
                html +=
                    "<div class='popupinfo-title' >" +
                    option.title +
                    "<el-icon class='pointer popupinfo-close' id='closePop'><close /></el-icon></div>";
                html += "<ul class='popupinfo-content'>";
                if (Object.keys(option.attrs).length > 0) {
                    for (var p in option.attrs) {
                        var _value = feature !== undefined ? feature.get(option.attrs[p].sxmc) : "";
                        if (option.attrs[p].callback) {
                            _value = option.attrs[p].callback(_value, feature);
                        }
                        if (
                            _value == "" ||
                            _value == "0" ||
                            _value == 0 ||
                            _value == null
                        ) {
                            _value = "";
                        }
                        html +=
                            "<li><span>" +
                            (option.attrs[p].sxbm || option.attrs[p].name) +
                            "：</span><span>" +
                            _value +
                            "</span></li>";
                    }
                }
            }
            html += "</ul><div class='popupinfo-poi'></div></div>";
            var para = document.createElement("p");
            para.innerHTML = html;
            var text = option.type ?
                document.getElementById(option.type) :
                document.getElementById(myMap._domid);
            text.appendChild(para);
            if (option.type) {
                document.getElementById("closePop").onclick = function() {
                    myMap[option.type].removeOverlay(
                        myMap[option.type].getOverlayById(_popupId_)
                    );
                };
            } else {
                document.getElementById("closePop").onclick = function() {
                    myMap.map1.removeOverlay(myMap.getPopupOverlay());
                };
            }
            var popupElement = document.getElementById(_popupId_);
            // var coord = evt ?
            //     evt.coordinate :
            //     getCenter(feature.getGeometry().getExtent());
            if(option.index == 0){
                var _overlay0 = new Overlay({
                    id: "popup_temp0",
                    position: option.tcPoint,
                    positioning: "bottom-center",
                    element: popupElement,
                    stopEvent: true,
                    autoPan: true,
                    autoPanAnimation: {
                        duration: 250,
                    },
                });
                if(myMap.map1.getOverlayById(_popupId_)){
                    myMap.map1.removeOverlay(myMap.map1.getOverlayById(_popupId_));

                }
                myMap.map1.addOverlay(_overlay0, option.type);
            }else{
                var _overlay1 = new Overlay({
                    id: "popup_temp1",
                    position: option.tcPoint,
                    positioning: "bottom-center",
                    element: popupElement,
                    stopEvent: true,
                    autoPan: true,
                    autoPanAnimation: {
                        duration: 250,
                    },
                });
                myMap[option.type].addOverlay(_overlay1, option.type);
            }
            popupElement.parentElement.style.zIndex = 1;
            // Status = Object.keys(option.attrs).length > 0 ? true : false;
            // myMap[option.type].getLayerById("popup_temp0")
            // setVisible(Status);
            return _overlay0 , _overlay1;
        },
        SearchWfsByLoc: function(layers, feature, buffer, callback) {
            if (typeof feature == "string") {
                var lonlat = (feature || "").split(",");
                var point = fromLonLat(
                    transform([lonlat[0], lonlat[1]], "EPSG:3857", "EPSG:4326")
                );
                feature = new Feature({
                    geometry: new Point(myMap.lonLatToWebMercator(point[0], point[1])),
                });
            }
            var geom;
            var bf;
            if (buffer) {
                var ol3 = new jsts.io.OL3Parser();
                ol3.inject(
                    Point,
                    LineString,
                    LinearRing,
                    Polygon,
                    MultiPoint,
                    MultiLineString,
                    MultiPolygon
                );
                bf = new Feature({
                    geometry: ol3.write(ol3.read(feature.getGeometry()).buffer(buffer)),
                });
                geom = bf.getGeometry();
            } else {
                geom = feature.getGeometry();
            }
            myMap.SearchWfsData(layers, geom, function(datas) {
                callback(datas, bf);
            });
        },
        //WFS查询
        SearchWfsData: function(layers, data, cl) {
            var filter;
            if (typeof data == "object") {
                filter = intersects("geom", data, "");
            } else {
                var datas = (data || "").split(":");
                filter = like(datas[0], "*" + datas[1] + "*");
            }
            var strs = new Array();
            strs = (layers || "").split(",");
            var featureRequest = new WFS().writeGetFeature({
                srsName: "EPSG:3857",
                featureNS: "p002",
                featurePrefix: "p002",
                featureTypes: strs,
                outputFormat: "application/json",
                filter: filter,
            });
            fetch("/geoserver/p002/ows", {
                    method: "POST",
                    body: new XMLSerializer().serializeToString(featureRequest),
                })
                .then(function(response) {
                    return response.json();
                })
                .then(function(json) {
                    var features = new GeoJSON().readFeatures(json);
                    if (cl) cl(features);
                });
        },
        //wfs属性查询
        SearchWfsFilter(layers, filter, cl) {
            var strs = new Array();
            strs = (layers || "").split(",");
            var featureRequest = new WFS().writeGetFeature({
                srsName: "EPSG:3857",
                featureNS: "p002",
                featurePrefix: "p002",
                featureTypes: strs,
                outputFormat: "application/json",
                filter: filter,
            });
            fetch("/geoserver/p002/wfs", {
                    method: "POST",
                    body: new XMLSerializer().serializeToString(featureRequest),
                })
                .then(function(response) {
                    return response.json();
                })
                .then(function(json) {
                    var features = new GeoJSON().readFeatures(json);
                    if (cl) cl(features);
                });
        },
        //图层数据叠加
        addDjLayer(url, type) {},
        //图层类型
        WMSImage(url, name) {
            var _layer = new ImageLayer({
                name: name,
                source: new ImageWMS({
                    ratio: 1,
                    url: url,
                    params: {
                        FORMAT: "image/png",
                        VERSION: "1.1.1",
                        STYLES: "",
                        LAYERS: name,
                    },
                }),
            });
            return _layer;
        },
        WMSTile: function(record) {
            var _layer = new TileLayer({
                name: record.url,
                id: record.id,
                source: new TileWMS({
                    ratio: 1,
                    url: record.url,
                    params: {
                        FORMAT: "image/png",
                        VERSION: "1.1.1",
                        tilad: true,
                        STYLES: "",
                        LAYERS: record.url,
                    },
                }),
            });
            myMap.addLayer(_layer);
        },
        XYZTile: function(record) {
            var imgsource = new XYZ({
                crossOrigin: "anonymous",
                url: record.url,
            });
            var untiled = new TileLayer({
                name: record.url,
                id: record.id,
                source: imgsource,
            });
            myMap.addLayer(untiled);
        },
        vector: function(url, name, style) {
            var _layer = new VectorLayer({
                name: name,
                declutter: false,
                visible: true,
                source: new VectorSource({
                    projection: "EPSG:3857",
                    url: url,
                }),
                style: style,
                zIndex: 1,
            });
            return _layer;
        },
        TILE: function(record, call, zoom) {
            var imgsource = new XYZ({
                crossOrigin: "anonymous",
                url: record.url,
            });
            var untiled = new TileLayer({
                name: record.url,
                id: record.bsm || record.id,
                source: imgsource,
                zIndex: (record.sort || 1) * -1,
            });
            if (call) {
                return untiled;
            }
            myMap.addLayer(untiled);
            //ZOOM
            if (zoom) {
                myMap.zoomToLayer(record.url);
            }
        },
        WMS: function(record, call, zoom) {
            var format = "image/png";
            var imgsource = new TileWMS({
                ratio: 1,
                url: BaseMap.geoImg,
                params: {
                    FORMAT: format,
                    VERSION: "1.1.1",
                    tilad: true,
                    STYLES: "",
                    LAYERS: record.url,
                },
            });
            var untiled = new TileLayer({
                name: record.url,
                id: record.bsm || record.id,
                source: imgsource,
                zIndex: (record.sort || 1) * -1,
            });
            if (call) {
                return untiled;
            }
            myMap.addLayer(untiled);
        },
        WMTS: function(record, call, zoom) {
            var imgsource = new XYZ({
                crossOrigin: "anonymous",
                url: record.url + `/tile/{z}/{y}/{x}`,
            });
            var untiled = new TileLayer({
                name: record.url,
                id: record.bsm || record.id,
                source: imgsource,
                zIndex: (record.sort || 1) * -1,
            });
            if (call) {
                return untiled;
            }
            myMap.addLayer(untiled);
            //ZOOM
            if (zoom) {
                myMap.zoomToLayer(record.url);
            }
        },
        //加载图层
        addProLayer: function(record, call, zoom) {
            var format = "image/png";
            var imgsource = new TileWMS({
                ratio: 1,
                url: BaseMap.geoImg,
                params: {
                    FORMAT: format,
                    VERSION: "1.1.1",
                    tilad: true,
                    STYLES: "",
                    LAYERS: record.url,
                },
            });
            var untiled = new TileLayer({
                name: record.url,
                id: record.id,
                source: imgsource,
                zIndex: (record.sort || 1) * -1,
            });
            if (call) {
                return untiled;
            }
            myMap.addLayer(untiled);
        },
        // 加载geojson数据
        addGeoJson: function(name, json) {
            let layer = myMap.getLayer(name);
            if (layer) {
                layer.getSource().clear();
            } else {
                myMap.addLayer(myMap.CreateVecLayer(name));
                layer = myMap.getLayer(name);
            }
            var features = new GeoJSON().readFeatures(json, {
                defaultDataProjection: "EPSG:4326",
                featureProjection: "EPSG:3857",
            });
            console.log(typeof json);
            layer.getSource().addFeatures(features);
            return layer;
        },
        zoomToFeatureJson: function(json) {
            var feature = new GeoJSON().readFeature(json, {
                defaultDataProjection: "EPSG:4326",
                featureProjection: "EPSG:3857",
            });
            var f = new Feature({
                geometry: feature.getGeometry(),
            });
            myMap.onLocation(f);
        },
        //定位
        onLocation(feature, zoom) {
            var show = 0;
            if (zoom === undefined) zoom = true;

            function sansuo() {
                myMap.tempLayers.dview_location_Layer.getSource().clear();
                if (show % 2 == 0) {
                    if (Array.isArray(feature)) {
                        myMap.tempLayers.dview_location_Layer
                            .getSource()
                            .addFeatures(feature);
                    } else {
                        myMap.tempLayers.dview_location_Layer
                            .getSource()
                            .addFeature(feature);
                        if (zoom)
                            myMap.zoomToextent(feature.getGeometry().getExtent(), 500);
                    }
                }
                if (show < 4) {
                    setTimeout(() => {
                        sansuo();
                    }, 200);
                }
                show++;
            }
            sansuo();
        },
    };
    return myMaps;
})();
export default myMap;