/*
 * @Descripttion: 定制飞行
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-08-08 17:18:39
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-08 17:55:14
 */
import * as Cesium from "cesium";
let OrbitFlightCameraExection = null;
/**
 * 绕点飞行公共接口
 */
export function OrbitFlightCamera(viewer) {
    if (OrbitFlightCameraExection) {
        ClearOrbitFlightCamera(viewer);
        return;
    }
    var ellipsoid = viewer.scene.globe.ellipsoid;
    var center = viewer.scene.pickPosition(new Cesium.Cartesian2(viewer.canvas.clientWidth / 2, (viewer.canvas.clientHeight / 2)));
    var carto_lt2 = ellipsoid.cartesianToCartographic(center);
    if (viewer.terrainProvider) {
        var allPoints = [carto_lt2];
        try {
            var promise = Cesium.sampleTerrain(viewer.terrainProvider, 15, allPoints);
            promise.then(function (updatedPositions) {
                updatedPositions[0].height = !updatedPositions[0].height ? 5000 : updatedPositions[0].height
                var cameraHeight = Math.ceil(viewer.camera.positionCartographic.height);
                var centerHeight = updatedPositions[0].height;
                var DiffZ = cameraHeight - centerHeight;
                var ellipsoid = viewer.scene.globe.ellipsoid;
                var cameraAngle = Math.abs(Cesium.Math.toDegrees(viewer.camera.pitch));
                var center = viewer.scene.pickPosition(new Cesium.Cartesian2(viewer.canvas.clientWidth / 2, (viewer.canvas.clientHeight / 2)));
                var carto_lt2 = ellipsoid.cartesianToCartographic(center);
                var position = Cesium.Cartesian3.fromDegrees(Cesium.Math.toDegrees(carto_lt2.longitude), Cesium.Math.toDegrees(carto_lt2.latitude), centerHeight);
                var distance = cameraHeight / Math.sin(cameraAngle * Math.PI / 180);
                var centerDistance = centerHeight / Math.sin(cameraAngle * Math.PI / 180);
                distance = distance - centerDistance;
                // 给定飞行一周所需时间
                var angle = 360 / 20;
                var startTime = Cesium.JulianDate.fromDate(new Date());
                var stopTime = Cesium.JulianDate.fromIso8601("2099-12-26");
                viewer.clock.startTime = startTime.clone(); // 开始时间
                viewer.clock.stopTime = stopTime.clone(); // 结速时间
                viewer.clock.currentTime = startTime.clone(); // 当前时间
                viewer.clock.clockRange = Cesium.ClockRange.CLAMPED; // 行为方式
                viewer.clock.clockStep = Cesium.ClockStep.SYSTEM_CLOCK; // 时钟设置为当前系统时间; 忽略所有其他设置。
                // 相机的当前heading
                var initialHeading = Cesium.Math.toDegrees(viewer.camera.heading);
                var initialPitch = viewer.camera.pitch;
                var initialRoll = viewer.camera.roll;
                OrbitFlightCameraExection = function CameraTimeExecution() {
                    // 当前已经过去的时间，单位s
                    var delTime = Cesium.JulianDate.secondsDifference(viewer.clock.currentTime, viewer.clock.startTime);
                    // var heading = Cesium.Math.toRadians(delTime * angle) - initialHeading;
                    var heading = initialHeading - delTime * angle;
                    if (heading < 0) {
                        heading = heading + Math.ceil(Math.abs(heading) / 360) * 360;
                    }
                    viewer.scene.camera.setView({
                        destination: center, // 点的坐标
                        orientation: {
                            heading: Cesium.Math.toRadians(heading),
                            pitch: initialPitch,
                            roll: initialRoll
                        }
                    });
                    viewer.scene.camera.moveBackward(distance);
                    if (Cesium.JulianDate.compare(viewer.clock.currentTime, viewer.clock.stopTime) >= 0) {
                        viewer.clock.onTick.removeEventListener(OrbitFlightCameraExection);
                    }
                };
                viewer.clock.onTick.addEventListener(OrbitFlightCameraExection);
            });
            // Cesium.when(promise, (updatedPositions) => {

            // })
        } catch (e) {

        }
    } else {
    }
}
export function ClearOrbitFlightCamera(viewer) {
    if (OrbitFlightCameraExection) {
        viewer.clock.onTick.removeEventListener(OrbitFlightCameraExection);
        OrbitFlightCameraExection = null;
    }
}