/*
 * @Descripttion: 绘制方法
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-08-05 15:41:37
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-10 16:38:14
 */
import * as Cesium from "cesium";
import * as turf from '@turf/turf'
import { GlobeTracker } from "@/utils/map/drawPlugin/GlobeTracker";
import { getPickPositionGlobe, setTooltip, removeAllEntities } from "@/utils/map/mapUtils";
import { Cartesian3_to_WGS84Arr } from "/src/utils/tools/CoordTransformate/CoordTransformate"

let tracker = undefined;
// 初始化绘制
export function initTracker(viewer) {
    if (!tracker) {
        tracker = new GlobeTracker(viewer);
    }
    return tracker;
}
// 清除绘制
export function clearTrack(viewer) {
    if (tracker) {
        tracker.clear();
        tracker = undefined;
    }
}
// 测量
export function layerMeasure(viewer, type) {
    let drawTracker = initTracker(viewer);
    removeAllEntities(viewer);
    switch (type) {
        case "distance":
            // 测距
            drawTracker.pickSpaceDistance(function (positions, rlt) { });
            break;
        case "area":
            // 测面
            drawTracker.pickArea(function (positions, rlt) { });
            break;
        case "stickDistance":
            // 贴地
            drawTracker.pickStickDistance(function (positions, rlt) { });
            break;
        case "height":
            // 测高
            measureHeight(viewer)
            break;
        default:
            break;
    }
}
var drawBackFun = null;
var drawBackSpecial = null;

export function DrawHelper(viewer, type) {
    let drawTracker = initTracker(viewer);
    switch (type) {
        case "drawPolygon":
            drawTracker.trackPolygon(function (positions) {
                drawBackFunHandle(viewer, positions, "polygon");
            });
            break;
        case "drawPolyline":
            drawTracker.trackPolyline(function (positions) {
                drawBackFunHandle(viewer, positions, "polyline");
            });
            break;
        case "drawRectangle":
            drawTracker.trackRectangle(function (positions) {
                drawBackFunHandle(viewer, positions, "rectangle");
            });
            break;
        case "drawCircle":
            drawTracker.trackCircle(function (positions) {
                drawBackFunHandle(viewer, positions, "circle");
            });
            break;
        case "drawPoint":
            drawTracker.trackPoint(function (position) {
                drawBackFunHandle(viewer, position, "point");
            }, null, true);
            break;
        case "drawLabelPoint":
            drawTracker.trackPoint(function (position) {
                drawBackFunHandle(viewer, position);
                layer.prompt({
                    title: '请输入标注内容'
                }, function (value, index, elem) {
                    tracker.clear();
                    layer.close(index);
                    viewer.entities.add({
                        id: getUUID(),
                        name: getUUID(),
                        position: position,
                        point: {
                            color: Cesium.Color.GREEN.withAlpha(1),
                            pixelSize: 5,
                            heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
                            outlineColor: Cesium.Color.YELLOW,
                            outlineWidth: 2,
                        },
                        label: {
                            text: value,
                            fillColor: Cesium.Color.fromCssColorString('#ff0000'),
                            font: '15px',
                            showBackground: false,
                            horizontalOrigin: Cesium.HorizontalOrigin.CENTER,
                            verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                            pixelOffset: new Cesium.Cartesian2(0, -30)
                        }
                    });
                });
            }, null, true);
            break;
        case "drawBufferLine":
            drawTracker.trackBufferLine(function (positions, radius) { });
            break;
        case "straightArrow":
            drawTracker.trackStraightArrow(function (positions) {
                drawBackFunHandle(viewer, positions);
            });
            break;
        case "attackArrow":
            drawTracker.trackAttackArrow(function (positions, custom) {
                drawBackFunHandle(viewer, positions);
            });
            break;
        case "pincerArrow":
            drawTracker.trackPincerArrow(function (positions, custom) {
                drawBackFunHandle(viewer, positions);
            });
            break;
        default:
            break;
    }
}

//绘制回调
export function drawBackFunHandle(viewer, positions, type) {
    //if (!drawBackFun || !type) { return;}
    if (!(positions instanceof Array)) {
        positions = [positions];
    }
    var drawPoints = Cartesian3_to_WGS84Arr(positions);
    var points = Array.from(drawPoints);
    points.push(points[0]);
    var area = 0;
    if (type != "point" && type != "polyline") {
        var polygon = turf.polygon([points]);
        area = turf.area(polygon);
    } else if (type == "point" && drawBackSpecial) {
        var entity = viewer.entities.add({
            id: "dlxz_" + drawBackSpecial,
            position: Cesium.Cartesian3.fromDegrees(points[0][0], points[0][1]),
            //position: positions,
            billboard: { //图标
                image: '/src/assets/images/map/' + drawBackSpecial + '.png',
                width: 50,
                height: 50,
                pixelOffset: new Cesium.Cartesian2(0, -20),   //偏移量
            },
        });
        //viewer.zoomTo(entity);
    }
    var thisResult = {
        prj: "EPSG:4326",
        type: type,
        // points: drawPoints //不闭合的
        points: points // 闭合的
    }
    thisResult["mode"] = "3D";
    thisResult["features"] = changeFormatToJson(thisResult.type, thisResult.points, thisResult.prj);
    thisResult["area"] = area;
    //TODO 绘制完成监听
    // window.parent.parent.postMessage({
    //     msg: "receiveDrawInfo",
    //     info: JSON.stringify(thisResult)
    // }, '*');
    // window.parent.postMessage({
    //     script: 'globleClearHandle()'
    // });
    if (drawBackFun) {
        drawBackFun(thisResult.features);
    }
}

export function changeFormatToJson(geomType, geomPoints, geomPrj) {
    if (geomPrj) {
        geomPrj = (geomPrj.split(":").length == 2 && (geomPrj.split(":")[0] == "EPSG" || geomPrj.split(":")[0] == "wkid")) ? geomPrj.split(":")[1] : geomPrj;
    }
    geomPrj = Number(geomPrj);
    var geometry = null;
    switch (geomType) {
        case "point":
        case "esriGeometryPoint":
            geomType = "esriGeometryPoint";
            geometry = { x: geomPoints[0][0], y: geomPoints[0][1] }
            break;
        case "polyline":
        case "esriGeometryPolyline":
            geomType = "esriGeometryPolyline";
            geometry = {
                paths: geomPoints
            };
            break;
        case "polygon":
        case "esriGeometryPolygon":
        default:
            geomType = "esriGeometryPolygon";
            geometry = {
                rings: [geomPoints]
            };
            break;
    }
    var thisResult1 = {
        displayFieldName: "",
        fieldAliases: {},
        geometryType: geomType,
        spatialReference: {
            wkid: geomPrj,
            latestWkid: geomPrj
        },
        fields: [],
        features: [{
            attributes: {},
            geometry: geometry
        }]
    }
    return thisResult1;
}
// 测高
export function measureHeight(viewer, handler) {
    var handler_g = handler = new Cesium.ScreenSpaceEventHandler(viewer.scene.canvas);
    var positions = [];
    var poly = null;
    // var tooltip = document.getElementById("toolTip");
    var tooltip = setTooltip(viewer)
    var height = 0;
    var cartesian = null;
    var floatingPoint;
    // tooltip.style.display = "block";
    tooltip.setVisible(true);

    handler.setInputAction(function (movement) {
        tooltip.showAt(movement.endPosition, "<p style='margin: 0'>单击开始，右击结束</p>", 3, -25);
        cartesian = getPickPositionGlobe(movement.endPosition, viewer);
        if (positions.length >= 2) {
            if (!Cesium.defined(poly)) {
                poly = new PolyLinePrimitive(positions);
            } else {
                positions.pop();
                positions.push(cartesian);
            }
            height = getHeight(positions);
        }
    }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);

    handler.setInputAction(function (movement) {
        tooltip.setVisible(false);
        cartesian = getPickPositionGlobe(movement.position, viewer);

        if (positions.length == 0) {
            positions.push(cartesian.clone());
            positions.push(cartesian);

            floatingPoint = viewer.entities.add({
                //parent: measure_entities,
                name: '高度',
                position: positions[0],
                point: {
                    pixelSize: 5,
                    color: Cesium.Color.RED,
                    outlineColor: Cesium.Color.WHITE,
                    outlineWidth: 2,
                    heightReference: Cesium.HeightReference.none
                },
                //label: {
                //    text: "0米",
                //    font: '18px sans-serif',
                //    fillColor: Cesium.Color.GOLD,
                //    style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                //    outlineWidth: 2,
                //    verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                //    pixelOffset: new Cesium.Cartesian2(20, -40)
                //}
            });
        }
    }, Cesium.ScreenSpaceEventType.LEFT_CLICK);

    handler.setInputAction(function (movement) {
        handler.destroy();
        // tooltip.style.display = "none";
        tooltip.setVisible(false);
        var textDisance = height + "米";
        var point1cartographic = Cesium.Cartographic.fromCartesian(positions[0]);
        var point2cartographic = Cesium.Cartographic.fromCartesian(positions[1]);
        var point_temp = Cesium.Cartesian3.fromDegrees(Cesium.Math.toDegrees(point1cartographic.longitude), Cesium.Math.toDegrees(point1cartographic.latitude), point2cartographic.height);
        viewer.entities.add({
            name: '直线距离',
            position: point_temp,
            point: {
                pixelSize: 5,
                color: Cesium.Color.RED,
                outlineColor: Cesium.Color.WHITE,
                outlineWidth: 2,
                heightReference: Cesium.HeightReference.none
            },
            label: {
                text: textDisance,
                font: '18px sans-serif',
                fillColor: Cesium.Color.GOLD,
                style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                outlineWidth: 2,
                verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                pixelOffset: new Cesium.Cartesian2(20, -20)
            }
        });
    }, Cesium.ScreenSpaceEventType.RIGHT_CLICK);

    function getHeight(_positions) {
        var cartographic = Cesium.Cartographic.fromCartesian(_positions[0]);
        var cartographic1 = Cesium.Cartographic.fromCartesian(_positions[1]);
        var height_temp = cartographic1.height - cartographic.height;
        return height_temp.toFixed(2);
    }

    var PolyLinePrimitive = (function () {
        function _(positions) {
            this.options = {
                //parent: measure_entities,
                name: '直线',
                polyline: {
                    show: true,
                    positions: [],
                    material: Cesium.Color.AQUA,
                    width: 2
                },
                ellipse: {
                    show: true,
                    // semiMinorAxis : 30.0,
                    // semiMajorAxis : 30.0,
                    // height: 20.0,
                    material: Cesium.Color.GREEN.withAlpha(0.5),
                    outline: true // height must be set for outline to display
                }
            };
            this.positions = positions;
            this._init();
        }

        _.prototype._init = function () {
            var _self = this;
            var _update = function () {
                var temp_position = [];
                temp_position.push(_self.positions[0]);
                var point1cartographic = Cesium.Cartographic.fromCartesian(_self.positions[0]);
                var point2cartographic = Cesium.Cartographic.fromCartesian(_self.positions[1]);
                var point_temp = Cesium.Cartesian3.fromDegrees(Cesium.Math.toDegrees(point1cartographic.longitude), Cesium.Math.toDegrees(point1cartographic.latitude), point2cartographic.height);
                temp_position.push(point_temp);
                return temp_position;
            };
            var _update_ellipse = function () {
                return _self.positions[0];
            };
            var _semiMinorAxis = function () {
                var point1cartographic = Cesium.Cartographic.fromCartesian(_self.positions[0]);
                var point2cartographic = Cesium.Cartographic.fromCartesian(_self.positions[1]);
                /**根据经纬度计算出距离**/
                var geodesic = new Cesium.EllipsoidGeodesic();
                geodesic.setEndPoints(point1cartographic, point2cartographic);
                var s = geodesic.surfaceDistance;
                return s;
            };
            var _height = function () {
                var height_temp = getHeight(_self.positions);
                return height_temp;
            };
            //实时更新polyline.positions
            this.options.polyline.positions = new Cesium.CallbackProperty(_update, false);
            this.options.position = new Cesium.CallbackProperty(_update_ellipse, false);
            this.options.ellipse.semiMinorAxis = new Cesium.CallbackProperty(_semiMinorAxis, false);
            this.options.ellipse.semiMajorAxis = new Cesium.CallbackProperty(_semiMinorAxis, false);
            this.options.ellipse.height = new Cesium.CallbackProperty(_height, false);
            viewer.entities.add(this.options);
        };

        return _;
    })();
}
/**
 * 全局绘制公共接口
 * @param drawList
 * @param callback
 * @param special 起点终点start/end
 * @constructor
 */
export function g_DrawFigure(viewer, drawList, callback, special) {
    if (special) {
        viewer.entities.removeById("dlxz_" + special);
    } else {
        removeAllEntities(viewer);
    }
    if (drawList && (drawList == "point" || drawList.toString().indexOf("point") > -1)) {
        DrawHelper(viewer, "drawPoint")
        drawBackSpecial = special;
    } else {
        DrawHelper(viewer, "drawPolygon")
    }
    drawBackFun = callback;
}