/*
 * @Descripttion: 图层管理
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-07-30 10:08:50
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-10 16:34:18
 */
import { sw_servicetype, LayerHelper } from "@/utils/map/layerHelper";
import { g_flyto } from "@/utils/map/fun/layerPosition";
/**
 * 加载默认图层
 * @return {*}
 */
export function show3DLayersMap(MapConfigLayers, viewer) {
    if (
        MapConfigLayers.maplayers &&
        MapConfigLayers.maplayers.length > 0
    ) {
        MapConfigLayers.maplayers.map((layerInfo) => {
            if (layerInfo.checked) {
                layerInfo.GUID = layerInfo.id;
                // layerInfo.NAME = layerInfo.name;
                layerInfo.NAME = "";
                layerInfo.URL = layerInfo.layerurl;
                var layerHelper = new LayerHelper(
                    layerInfo,
                    false,
                    viewer
                );
                layerHelper.addLayer();
            }
        });
    }
}
/**
 * @Desc: 加载指定图层
 * @param {*} layerinfo
 * @param {*} visible
 * @param {*} isbasemap
 * @return {*}
 */
export function addService({ layerinfo = {}, visible = true, isbasemap = false }, viewer) {
    if (!layerinfo) return;
    layerinfo["URL"] = layerinfo["ADDRESS"];
    var layerHelp = new LayerHelper(layerinfo, isbasemap, viewer);
    layerHelp.addLayer(visible);
}
/**
 * @Desc: 加载多个图层
 * @param {*} layerinfos
 * @return {*}
 */
export function addServices(layerinfos, viewer) {
    if (layerinfos && layerinfos.length > 0) {
        for (var i = 0; i < layerinfos.length; i++) {
            var layerinfo = layerinfos[i];
            this.addService({
                layerinfo: layerinfo.layerinfo,
                visible: layerinfo.visible,
                isbasemap: layerinfo.isbasemap
            }, viewer
            );
        }
    }
}
/**
 * 透明度控制
 * @param id guid
 * @param opacity  1-0
 */
export function globleSetLayerOpacity(id, opacity, viewer) {
    // opacity = opacity ? parseFloat(opacity) : 1;
    var thisLayerInfo = window.treeList[id];
    var layerHelp = new LayerHelper(thisLayerInfo, false, viewer);
    layerHelp.changeLayerTransparent(id, thisLayerInfo, opacity);
}
/**
 * 调整图层顺序
 * @param id  guid
 * @param type  up / down
 */
export function globleSetLayerIndex(thisLayerInfo, type, viewer) {

    var layerHelp = new LayerHelper(thisLayerInfo, false, viewer);
    if (type == "up") {
        layerHelp.moveToUp();
    } else if (type == "down") {
        layerHelp.moveToDown();
    }
}
//定位到图层位置
export function flyTo3DViewByLayerInfo(id, viewer) {
    //获取要定位的图层信息
    var locationLayer = window.treeList[id]
        ? window.treeList[id]
        : window.treeList;
    if (locationLayer["SERVICETYPE"] === sw_servicetype.sw_oblique) {
        return viewer.zoomTo(window.treeList[id].imageryLayer);
    } else if (
        locationLayer["SERVICETYPE"] === sw_servicetype.sw_terrain
    ) {
        return null;
    }
    var positions = locationLayer.POSITION;
    if (positions != "") {
        positions = positions.split(",");
    }

    if (positions.length < 3) {
        //没有缩放级别时，补空
        positions.push("");
    }

    //图层上移
    //var layerHelp = new LayerHelper(locationLayer);
    //var thisLayer = layerHelp.GetLayer();
    //layerHelp.moveToUp();

    if (positions != "") {
        var height;
        if (positions[2]) {
            height = parseFloat(positions[2]) + 1000;
        } else {
            height = Math.ceil(
                viewer.camera.positionCartographic.height
            );
        }
        g_flyto(viewer, { lon: parseFloat(positions[0]), lat: parseFloat(positions[1]), height: height });
    }
}