/*
 * @Descripttion: 图层要素渲染
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-08-10 16:33:54
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-10 17:57:35
 */
import * as Cesium from "cesium";
import { LayerHelper } from "@/utils/map/layerHelper";
import { removeAllEntities } from "@/utils/map/mapUtils";

/**
 * 绘制分析范围
 * @param params
 */
export function addAnalyseExtent(viewer, params) {
    removeAllEntities(viewer);
    var id = "c45B99afF137462FbfB85ffC01AddfA6";
    // params = params || { "displayFieldName": "", "fieldAliases": {}, "geometryType": "esriGeometryPolygon", "spatialReference": { "wkid": 4326, "latestWkid": 4326 }, "fields": [], "features": [{ "attributes": {}, "geometry": { "rings": [[[108.68490335673884, 40.03110255792514, 0], [108.68338318114769, 39.93183352900264, 0], [108.75865006380822, 39.90653515582441, 0], [108.78511526888586, 39.93984386540289, 0], [108.81590456454865, 39.99891644965434, 0], [108.75135444605658, 40.06876811971662, 0], [108.73095374976525, 40.088455735067214, 0], [108.73095374976525, 40.088455735067214, 0]]] } }] }
    if (typeof (params) == "string") {
        params = JSON.parse(params)
    }
    if (params && params.features && params.features[0].geometry.rings[0]) {
        var points = params.features[0].geometry.rings[0];
        var allPoints = [];
        for (var i = 0; i < points.length; i++) {
            allPoints.push(points[i][0]);
            allPoints.push(points[i][1]);
        }
        var polylinePoints = Array.from(allPoints);
        polylinePoints.push(allPoints[0]);
        polylinePoints.push(allPoints[1]);
        var entity = viewer.entities.add({
            name: 'analyse',
            id: 'analyse_' + id,
            polygon: {
                hierarchy: Cesium.Cartesian3.fromDegreesArray(allPoints),
                material: Cesium.Color.WHITE.withAlpha(0.2),
                outline: true,
                outlineColor: Cesium.Color.RED.withAlpha(1)
            },
            polyline: {
                clampToGround: true,
                material: Cesium.Color.RED.withAlpha(1),
                positions: Cesium.Cartesian3.fromDegreesArray(polylinePoints),
                width: 2
            }
        });
        viewer.flyTo(entity);
    } else {
        console.log("必要参数为传递！");
    }
}
/**
 * 应用分析矢量切片服务
 * 根据分析结果数据名称进行数据渲染
 * @param resultName 不传参则为清除展示图层
 * @param type 数据类型
 */
export function overlayVectorFeature(viewer, resultName, type) {
    var path = import.meta.env.VITE_Vector_Path;
    console.log(path);
    var id = "c45B99afF137462FbfB85ffC01AddfA8";
    if (window.treeList[id]) {
        var imageryLayer = window.treeList[id].imageryLayer
        viewer.scene.imageryLayers.remove(imageryLayer);
        window.treeList[id].imageryLayer = null
    }
    if (!resultName) {
        console.log("必要参数未传递！");
        return;
    }
    var layerparam = {
        "GUID": id,
        "NAME": "",
        "SERVICETYPE": "tile",
        "SOURCE": "pbftile",
        "DATAURI": path + "\\" + resultName + ".shp",
        "LAYERNAME": resultName,
        "SPATIALREF": "EPSG:4326"
    };
    var layer = new LayerHelper(layerparam, false, viewer);
    layer.addLayer(true);
}
/**
 * 矢量数据渲染geojson
 * @param url
 */
export function geojsonLayer(viewer, url) {
    var id = "c45B99afF137462FbfB85ffC01AddfA";
    if (window.treeList[id]) {
        globleStreamerpath.removeprimitive(id);
        window.treeList[id].imageryLayer = null
    }
    if (url) {
        var layerparam = {
            "GUID": id,
            "NAME": "",
            "URL": url,
            "SERVICETYPE": "geojson",
            "ALIAS": "geodata"
        };
        var layer = new LayerHelper(layerparam, false, viewer);
        layer.addLayer(true);
    }
}
