/*
 * @Descripttion: 图层查询
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-08-01 16:25:45
 * @LastEditors: 郭亚伟 2241880990@qq.com
 * @LastEditTime: 2022-11-09 14:34:15
 */
import store from '@/store'
import axios from 'axios';
import * as Cesium from "cesium";
import { getPickPositionGlobe, handleToGeojson, removeAllEntities, GetViewExtent, setTooltip, setScreenSpaceEventHandler, clearScreenSpaceEventHandler } from "@/utils/map/mapUtils";
import { sw_servicetype, sw_source } from "@/utils/map/layerHelper";
import mapRequest from "@/utils/map/mapRequest";
import { getFieldsInfo } from "@/api/main/index.js";
import { addProjectionTransform } from "/src/utils/tools/CoordTransformate/CoordTransformate"

let { LayerFieldFilterInfo } = window.mapConfig
let QueryHandler = null;
let sampleLabel = null; // 点选标注
/**
 * @Desc: 
 * @Author: 牛夏蕊
 * @Date: 2022-08-03 17:19:45
 * @param {*} viewer
 * @param {*} type 1:点选2：框选 5：i查询 
 * @param {*} callback
 * @return {*}
 */
export function layerQuery(viewer, type, callback) {
    let tooltip = setTooltip(viewer);
    removeAllEntities(viewer);
    // identifyClear();
    clearScreenSpaceEventHandler();
    removeSampleLabel();
    var cartesian = null;
    let QueryHandler = setScreenSpaceEventHandler(viewer);
    // QueryHandler = new Cesium.ScreenSpaceEventHandler(viewer.scene.canvas);
    var ellipsoid = viewer.scene.globe.ellipsoid;
    switch (type) {
        case "1":
            QueryHandler.setInputAction(function (movement) {
                //通过指定的椭球或者地图对应的坐标系，将鼠标的二维坐标转换为对应椭球体三维坐标
                cartesian = getPickPositionGlobe(movement.position, viewer);
                if (cartesian) {
                    //将笛卡尔坐标转换为地理坐标
                    var cartographic = ellipsoid.cartesianToCartographic(cartesian);
                    //将弧度转为度的十进制度表示
                    var longitudeString = Cesium.Math.toDegrees(cartographic.longitude);
                    var latitudeString = Cesium.Math.toDegrees(cartographic.latitude);
                    var point = longitudeString + ',' + latitudeString;
                    removeAllEntities(viewer)
                    queryLayerInfoByPoint(viewer, { point: point, type: 1 }, function (result) {
                        callback(result)
                    })
                    tooltip.setVisible(false);
                }
            }, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            break;
        case "2":
            let activeShapePoints = [];
            let activeShape;
            let floatingPoint;
            let drawingMode = 'rectangle';
            viewer.cesiumWidget.screenSpaceEventHandler.removeInputAction(Cesium.ScreenSpaceEventType.LEFT_DOUBLE_CLICK);
            //鼠标左键
            QueryHandler.setInputAction(function (event) {
                let earthPosition = getPickPositionGlobe(event.position, viewer);
                if (Cesium.defined(earthPosition)) {
                    if (activeShapePoints.length === 0) {
                        floatingPoint = createPoint(viewer, earthPosition);
                        activeShapePoints.push(earthPosition);
                        let dynamicPositions = new Cesium.CallbackProperty(function () {
                            if (drawingMode === 'polygon') {
                                return new Cesium.PolygonHierarchy(activeShapePoints);
                            }
                            return activeShapePoints;
                        }, false);
                        activeShape = drawShape(viewer, dynamicPositions); //绘制动态图
                    }
                    activeShapePoints.push(earthPosition);
                    createPoint(viewer, earthPosition);
                }
            }, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            //鼠标移动
            QueryHandler.setInputAction(function (event) {
                tooltip.setVisible(true);
                var wp = event.endPosition;
                if (!Cesium.defined(wp)) {
                    return;
                }
                // var position = event.endPosition;
                // if (!Cesium.defined(position)) {
                //     return;
                // }
                var num = activeShapePoints.length;
                if (num < 1) {
                    tooltip.showAt(wp, "<p style='margin:0'>单击选择起点</p>")
                    return;
                }
                tooltip.showAt(wp, "<p style='margin: 0'>右击结束绘制</p>");
                if (Cesium.defined(floatingPoint)) {
                    var newPosition = getPickPositionGlobe(event.endPosition, viewer);
                    if (Cesium.defined(newPosition)) {
                        floatingPoint.position.setValue(newPosition);
                        activeShapePoints.pop();
                        activeShapePoints.push(newPosition);
                    }
                }
            }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
            QueryHandler.setInputAction(function (event) {
                var point1 = [activeShapePoints[0].x, activeShapePoints[0].y, activeShapePoints[0].z];
                var point2 = [activeShapePoints[1].x, activeShapePoints[1].y, activeShapePoints[1].z];
                var pointsArr = [point1, point2];
                pointsArr = mapProjectionToWGS84_2(viewer, pointsArr);
                // terminateShape();
                activeShapePoints.pop(); //去除最后一个动态点
                if (activeShapePoints.length) {
                    drawShape(viewer, activeShapePoints); //绘制最终图
                }
                viewer.entities.remove(floatingPoint); //去除动态点图形（当前鼠标点）
                viewer.entities.remove(activeShape); //去除动态图形
                floatingPoint = undefined;
                activeShape = undefined;

                var drawPoint1 = [pointsArr[0].longitude, pointsArr[0].latitude];
                var drawPoint2 = [pointsArr[1].longitude, pointsArr[0].latitude];
                var drawPoint3 = [pointsArr[1].longitude, pointsArr[1].latitude];
                var drawPoint4 = [pointsArr[0].longitude, pointsArr[1].latitude];
                pointsArr = [drawPoint1, drawPoint2, drawPoint3, drawPoint4, drawPoint1];
                var points = pointArrsToStringForQuery(pointsArr);
                removeAllEntities(viewer);
                queryLayerInfoByPoint(viewer, { point: points, type: 3 }, function (result) {
                    callback(result)
                })
                tooltip.setVisible(false);
                activeShapePoints = []
            }, Cesium.ScreenSpaceEventType.RIGHT_CLICK);
            break;
        case "5":
            QueryHandler.setInputAction(function (movement) {
                removeSampleLabel()
                removeAllEntities(viewer);
                //通过指定的椭球或者地图对应的坐标系，将鼠标的二维坐标转换为对应椭球体三维坐标
                cartesian = getPickPositionGlobe(movement.position, viewer);
                if (cartesian) {
                    //将笛卡尔坐标转换为地理坐标
                    var cartographic = ellipsoid.cartesianToCartographic(cartesian);
                    //将弧度转为度的十进制度表示
                    var longitudeString = Cesium.Math.toDegrees(cartographic.longitude);
                    var latitudeString = Cesium.Math.toDegrees(cartographic.latitude);
                    var point = longitudeString + ',' + latitudeString;
                    FilterQueryLayerInfoByPoint(viewer, point, 1, function (result) {
                        if (result.length > 0) {
                            let str = ''
                            result.map((item, index) => {
                                let key = item.key;
                                let val = item.value;
                                if (index == 0) {
                                    str = `${key} : ${val}`;
                                } else {
                                    str += `</br>${key} : ${val}`;
                                }
                            })
                            sampleLabel = new xt3d.PointObject.SampleLabel(viewer, cartesian, str);
                            // sampleLabel = new xt3d.PointObject.DynamicDivLabel(viewer, cartesian, str);
                        }
                    });

                    tooltip.setVisible(false);
                }
            }, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            break;
        default:
            break;
    }
    if (type == "1" || type == "5") {
        //鼠标移动
        QueryHandler.setInputAction(function (event) {
            tooltip.setVisible(true);
            var wp = event.endPosition;
            if (!Cesium.defined(wp)) {
                return;
            }
            // if (cartesian == null) {
            tooltip.showAt(wp, "<p style='margin: 0'>单击选择位置</p>")
            // }
            var position = getPickPositionGlobe(wp, viewer);
            if (!Cesium.defined(position)) {
                return;
            }
        }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
    }
}

export function queryLayerInfoByPoint(viewer, obj, callback) {
    let point = obj.point;
    let type = obj.type;
    let layerIds = obj.layerIds;
    let layerIdList = store.state.resourceTree.checkedNode;
    let result;
    let results = [];
    if (!layerIds) {
        layerIds = []
        for (let i = 0; i < layerIdList.length; i++) {
            let layerId = layerIdList[i];
            let layerItem = window.treeList[layerId].imageryLayer;
            if (layerItem["NAME"] && layerItem["SERVICETYPE"] != sw_servicetype.sw_terrain && layerItem["SERVICETYPE"] != sw_servicetype.sw_oblique && layerItem["SOURCE"] != sw_source.sw_mongodb) {
                layerIds.push(layerId);
            }
        }
    } else if (typeof (layerIds) == "string") {
        layerIds = [layerIds];
    }

    if (layerIds.length == 0) {
        callback("请先打开图层再查询");
        clearScreenSpaceEventHandler();
        // identifyClear();
        return;
    }
    let flag = 0
    for (let i = 0; i < layerIds.length; i++) {
        let layerItem = window.treeList[layerIds[i]].imageryLayer;
        let queryUrl = "";
        let params = {};
        let layerUrl = layerItem["URL"];
        switch (layerItem["SERVICETYPE"]) {
            case sw_servicetype.sw_tile://tile
                switch (layerItem["SOURCE"]) {
                    case sw_source.sw_arcgis://arcgis
                    case sw_source.sw_arcgistile://arcgis
                        queryUrl = layerUrl + '/identify';
                        params = {
                            sr: 4326,
                            layers: 'visible',
                            tolerance: 3,
                            mapExtent: GetViewExtent(viewer).toString(),
                            imageDisplay: viewer.scene.canvas.width + ',' + viewer.scene.canvas.height + ',96',
                            returnGeometry: true,
                            returnFieldName: true,
                            f: 'json'
                        }
                        if (type == 1 || type == 2) {//point
                            params["geometry"] = JSON.stringify({ "x": parseFloat(point.split(",")[0]), "y": parseFloat(point.split(",")[1]) });
                            params["geometryType"] = 'esriGeometryPoint'
                        } else if (type == 3) {//polygon
                            // params["geometry"] = JSON.stringify({
                            //     "rings": [pointStringsToArrayForQuery(point)]
                            // });
                            params["geometry"] = encodeURIComponent(JSON.stringify({
                                "rings": [pointStringsToArrayForQuery(point)]

                            }));
                            params["geometryType"] = 'esriGeometryPolygon'

                        } else if (type == 4) {//polgline
                            params["geometry"] = encodeURIComponent(JSON.stringify({
                                "paths": [pointStringsToArrayForQuery(point)]
                            }));
                            params["geometryType"] = 'esriGeometryPolyline'
                        }
                        break;
                    case sw_source.sw_wms:
                        if (type == 1 || type == 2) {
                            queryUrl = layerUrl + "/ows?service=WFS&version=1.0.0&request=GetFeature&typeName=" + layerInfo.layerid + "&outputFormat=application/json&srsName=" + (layerInfo.srsName ? layerInfo.srsName : "EPSG:3857") + "&filter=<Filter xmlns='http://www.opengis.net/ogc' xmlns:gml='http://www.opengis.net/gml'><Intersects><PropertyName>the_geom</PropertyName><gml:Point srsName='EPSG:4326'><gml:coordinates>" + point + "</gml:coordinates></gml:Point></Intersects></Filter>";
                        } else if (type == 3) {
                            queryUrl = layerUrl + "/ows?service=WFS&version=1.0.0&request=GetFeature&typeName=" + layerInfo.layerid + "&outputFormat=application/json&srsName=" + (layerInfo.srsName ? layerInfo.srsName : "EPSG:3857") + "&filter=<Filter xmlns='http://www.opengis.net/ogc' xmlns:gml='http://www.opengis.net/gml'><Intersects><PropertyName>the_geom</PropertyName><gml:Polygon srsName='EPSG:4326'><gml:outerBoundaryIs><gml:LinearRing><gml:coordinates>" + point + "</gml:coordinates> </gml:LinearRing> </gml:outerBoundaryIs> </gml:Polygon></Intersects></Filter>";
                        } else if (type == 4) {
                            queryUrl = layerUrl + "/ows?service=WFS&version=1.0.0&request=GetFeature&typeName=" + layerInfo.layerid + "&outputFormat=application/json&srsName=" + (layerInfo.srsName ? layerInfo.srsName : "EPSG:3857") + "&filter=<Filter xmlns='http://www.opengis.net/ogc' xmlns:gml='http://www.opengis.net/gml'><Intersects><PropertyName>the_geom</PropertyName><gml:Polyline srsName='EPSG:4326'><gml:outerBoundaryIs><gml:LinearRing><gml:coordinates>" + point + "</gml:coordinates> </gml:LinearRing> </gml:outerBoundaryIs> </gml:Polyline></Intersects></Filter>";
                        }
                        break;
                    default:
                        break;
                }

                break;
            case sw_servicetype.vector://vector

                break;
            default:
                break;

        }
        // 发送请求
        mapRequest({
            url: queryUrl,
            method: 'get',
            params: params
        }).then((res) => {
            if (typeof (res) === "string") {
                res = JSON.parse(res)
            }
            result = handleToGeojson(res.results, "arcgis");
            if (result && result.features.length > 0) {
                // addQueryFeatures(viewer, result.features);
                results.push({
                    name: layerItem["NAME"],
                    id: layerIds[i],
                    data: result
                });
            }
            flag++;
            if (flag === layerIds.length) {
                callback(results)
            }
        })
    }
}

// i查询，过滤查询
export function FilterQueryLayerInfoByPoint(viewer, point, type, callback) {
    let layerIds = store.state.resourceTree.checkedNode;
    let result;
    if (layerIds.length == 0) {
        callback([]);
        // identifyClear();
        clearScreenSpaceEventHandler();
        removeSampleLabel();
        this.$modal.msgWarning('请先打开图层再查询')	
        return;
    }
    if (layerIds.length > 0) {
        //只查询最顶层图层信息
        let layerItem = window.treeList[layerIds[0]].imageryLayer;
        let queryUrl = "";
        let params = {};
        let layerUrl = layerItem["URL"];
        let layerName = layerItem["NAME"];
        switch (layerItem["SERVICETYPE"]) {
            case sw_servicetype.sw_tile: //tile
                switch (layerItem["SOURCE"]) {
                    case sw_source.sw_arcgis: //arcgis
                    case sw_source.sw_arcgistile: //arcgis
                        queryUrl = layerUrl + '/identify';
                        params = {
                            sr: 4326,
                            layers: 'visible',
                            tolerance: 3,
                            mapExtent: GetViewExtent(viewer).toString(),
                            imageDisplay: viewer.scene.canvas.width + ',' + viewer.scene.canvas.height + ',96',
                            returnGeometry: true,
                            returnFieldName: true,
                            f: 'json'
                        }
                        if (type == 1) { //point
                            params["geometry"] = JSON.stringify({
                                "x": parseFloat(point.split(",")[0]),
                                "y": parseFloat(point.split(",")[1])
                            });
                            params["geometryType"] = 'esriGeometryPoint'
                        }
                        break;
                    case sw_source.sw_wms:
                        if (type == 1) {
                            queryUrl = layerUrl + "/ows?service=WFS&version=1.0.0&request=GetFeature&typeName=" + layerInfo.layerid + "&outputFormat=application/json&srsName=" + (layerInfo.srsName ? layerInfo.srsName : "EPSG:3857") + "&filter=<Filter xmlns='http://www.opengis.net/ogc' xmlns:gml='http://www.opengis.net/gml'><Intersects><PropertyName>the_geom</PropertyName><gml:Point srsName='EPSG:4326'><gml:coordinates>" + point + "</gml:coordinates></gml:Point></Intersects></Filter>";
                        }
                        break;
                    default:
                        break;
                }

                break;
            case sw_servicetype.vector: //vector

                break;
            default:
                break;

        }
        // 发送请求
        mapRequest({
            url: queryUrl,
            method: 'get',
            params: params
        }).then((res) => {
            if (typeof (res) === "string") {
                res = JSON.parse(res);
            }
            result = handleToGeojson(res.results, "arcgis");
            if (result && result.features.length > 0) {
                addHighlightedFeatures(viewer, result.features, "polyline");
                filterQueryResult(result.features, layerIds[0], function (e) {
                    callback(e);
                });
            }
        })
    }
}
/**
 * 通过配置文件过滤，只显示预先配置好的字段信息
 * @param {*} features 
 */
export function filterQueryResult(features, layerId, callback) {
    let properties = features[0].properties,
        result = [];
    if (!LayerFieldFilterInfo[layerId]) {
        this.$modal.msgWarning('暂未配置属性字段！')	
    } else {
        getFieldsInfo({
            layerGuid: layerId,
        }).then((res) => {
            res = typeof res == "string" ? JSON.parse(res) : res;
            LayerFieldFilterInfo[layerId].map(function (item) {
                let resultInfo = {};
                let value = !properties[item.fieldName] ? "" : properties[item.fieldName]
                resultInfo.value = value;
                let alias = item.alias;
                // 优先使用配置文件里的别名
                if (alias && alias != "") {
                    resultInfo.key = item.alias;
                } else {
                    //属性字段解译
                    let obj = {}
                    if (res.data && res.data.length > 0) {
                        obj = res.data.find((i) => i.fieldname == item.fieldName)
                    }
                    if (Object.keys(obj).length == 0) {
                        resultInfo.key = item.fieldName;
                    } else {
                        resultInfo.key = obj.fieldalias;
                    }
                }
                result.push(resultInfo)
            });
            callback(result);
        });
    }
}
/**
 * 高亮显示要素
 * @param {*} features 
 * @param {string} geomtype "point","polyline","polygon"
 */
export function addHighlightedFeatures(viewer, features, geomtype) {
    for (var m = 0; m < features.length; m++) {
        var geom = features[m].geometry;
        // var geomtype = geom.type.toLowerCase();
        if (geomtype.indexOf("polygon") > -1) {
            var points = [];
            for (var i = 0; i < geom.coordinates[0][0].length; i++) {
                points.push(geom.coordinates[0][0][i][0]);
                points.push(geom.coordinates[0][0][i][1]);
            }
            viewer.entities.add({
                polygon: {
                    hierarchy: Cesium.Cartesian3.fromDegreesArray(points),
                    material: Cesium.Color.RED.withAlpha(0.5)
                }
            });
        } else if (geomtype.indexOf("polyline") > -1) {
            var points = [];
            for (var i = 0; i < geom.coordinates[0][0].length; i++) {
                points.push(geom.coordinates[0][0][i][0]);
                points.push(geom.coordinates[0][0][i][1]);
            }
            viewer.entities.add({
                polyline: {
                    clampToGround: true,
                    positions: Cesium.Cartesian3.fromDegreesArray(points),
                    material: Cesium.Color.AQUA.withAlpha(1), // Cesium.Color.BLUE.withAlpha(0.5) ,
                    width: 3,
                    //distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, updatedPositions[m + 4].height + 12000)
                }
            });
        } else if (geomtype.indexOf("point") > -1) {
            viewer.entities.add({
                position: Cesium.Cartesian3.fromDegrees(geom.coordinates[0], geom.coordinates[1]),
                point: {
                    color: Cesium.Color.RED.withAlpha(0.5),
                    pixelSize: 10,
                    heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
                    outlineColor: Cesium.Color.YELLOW,
                    outlineWidth: 2,
                }
            });
        }
    }
}

//绘制点
export function createPoint(viewer, worldPosition) {
    var point = viewer.entities.add({
        position: worldPosition,
        point: {
            color: Cesium.Color.WHITE,
            pixelSize: 5,
            heightReference: Cesium.HeightReference.CLAMP_TO_GROUND
        }
    });
    return point;
}
//绘制形状
export function drawShape(viewer, positionData) {
    //当positionData为数组时绘制最终图，如果为function则绘制动态图
    var arr = typeof positionData.getValue === 'function' ? positionData.getValue(0) : positionData;
    var shape = viewer.entities.add({
        name: 'Blue translucent, rotated, and extruded ellipse with outline',
        rectangle: {
            coordinates: new Cesium.CallbackProperty(function () {
                var obj = Cesium.Rectangle.fromCartesianArray(arr);
                return obj;
            }, false),
            material: Cesium.Color.RED.withAlpha(0.5)
        }
    });
    return shape;
}
export function mapProjectionToWGS84_2(viewer, pointArr) {
    var thisResult = [];
    for (var i = 0; i < pointArr.length; i++) {
        var ellipsoid = viewer.scene.globe.ellipsoid;
        if (pointArr[i].length === 3)
            var cartesian3 = new Cesium.Cartesian3(pointArr[i][0], pointArr[i][1], pointArr[i][2]);
        else
            var cartesian3 = new Cesium.Cartesian3(pointArr[i][0], pointArr[i][1], 0);
        var cartographic = ellipsoid.cartesianToCartographic(cartesian3);
        var lat = Cesium.Math.toDegrees(cartographic.latitude);
        var lng = Cesium.Math.toDegrees(cartographic.longitude);
        var alt = cartographic.height;
        var obj = { longitude: lng, latitude: lat, height: alt };
        thisResult.push(obj);
    }
    return thisResult;
}
export function pointArrsToStringForQuery(pointArrs) {
    var html = ''
    for (var i = 0; i < pointArrs.length; i++) {
        html += pointArrs[i][0] + ',' + pointArrs[i][1] + ' '
    }
    return html.substring(0, html.length - 1)
}

export function pointStringsToArrayForQuery(string) {
    var arr = [];
    for (var i = 0; i < string.split(" ").length; i++) {
        arr.push([string.split(" ")[i].split(",")[0], string.split(" ")[i].split(",")[1]]);
    }
    return arr;
}

/**
 * @Desc: 查询结果高亮显示
 * @param {*} viewer
 * @param {*} features
 * @return {*}
 */
export function addQueryFeatures(viewer, features, inSR) {
    if (!inSR) {
        inSR = "EPSG:" + features[0].arcgis.spatialReference.latestWkid; //3857
    }
    var outSR = "EPSG:4326"
    for (var m = 0; m < features.length; m++) {
        var geom = features[m].geometry;
        var geomtype = geom.type.toLowerCase();
        if (geomtype.indexOf("polygon") > -1) {
            var points = [];
            addProjectionTransform(geom.coordinates[0][0], inSR, outSR, true, false, function (e) {
                let coordinates = e.result
                for (var i = 0; i < coordinates.length; i++) {
                    var lon = coordinates[i][0];
                    var lat = coordinates[i][1];
                    points.push(lon);
                    points.push(lat);
                }
                viewer.entities.add({
                    polygon: {
                        hierarchy: Cesium.Cartesian3.fromDegreesArray(points),
                        material: Cesium.Color.RED.withAlpha(0.5)
                    }
                });
            });
        } else if (geomtype.indexOf("polyline") > -1) {
            var points = [];
            addProjectionTransform(geom.coordinates[0][0], inSR, outSR, true, false, function (e) {
                let coordinates = e.result
                for (var i = 0; i < coordinates.length; i++) {
                    var lon = coordinates[i][0];
                    var lat = coordinates[i][1];
                    points.push(lon);
                    points.push(lat);
                }
                viewer.entities.add({
                    polyline: {
                        positions: Cesium.Cartesian3.fromDegreesArray(points),
                        material: Cesium.Color.RED.withAlpha(0.5)
                    }
                });
            })
        } else if (geomtype.indexOf("point") > -1) {
            addProjectionTransform(geom.coordinates, inSR, outSR, true, false, function (e) {
                let coordinates = e.result
                viewer.entities.add({
                    position: Cesium.Cartesian3.fromDegrees(coordinates[0], coordinates[1]),
                    point: {
                        color: Cesium.Color.RED.withAlpha(0.5),
                        pixelSize: 10,
                        heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
                        outlineColor: Cesium.Color.YELLOW,
                        outlineWidth: 2,
                    }
                });
            })
        }
    }
}
/**
 * @Desc: 选中结果高亮显示
 * @param {*} geom
 * @param {*} isAllAttrQuery
 * @return {*}
 */
export function addSelectedFeatures(viewer, geom, inSR) {
    var outSR = "EPSG:4326"
    viewer.entities.removeById("selected_entity");
    var geomtype = geom.type.toLowerCase();
    var entity = null;
    var destination;
    if (geomtype.indexOf("polygon") > -1) {
        var points = [];
        addProjectionTransform(geom.coordinates[0][0], inSR, outSR, true, false, function (e) {
            let coordinates = e.result
            for (var i = 0; i < coordinates.length; i++) {
                var lon = coordinates[i][0];
                var lat = coordinates[i][1];
                points.push(lon);
                points.push(lat);
            }
            destination = Cesium.Cartesian3.fromDegreesArray(points);
            entity = viewer.entities.add({
                id: 'selected_entity',
                name: 'selected_entity',
                polygon: {
                    hierarchy: destination,
                    material: Cesium.Color.GREEN.withAlpha(0.7)
                }
            });
            if (entity) {
                viewer.flyTo(entity, {
                    offset: {
                        heading: Cesium.Math.toRadians(0.0),
                        pitch: Cesium.Math.toRadians(-90),
                        range: 0
                    }
                });
            }
        });
    } else if (geomtype.indexOf("polyline") > -1) {
        var points = [];
        addProjectionTransform(geom.coordinates[0][0], inSR, outSR, true, false, function (e) {
            let coordinates = e.result
            for (var i = 0; i < coordinates.length; i++) {
                var lon = coordinates[i][0];
                var lat = coordinates[i][1];
                points.push(lon);
                points.push(lat);
            }
            destination = Cesium.Cartesian3.fromDegreesArray(points);
            entity = viewer.entities.add({
                id: 'selected_entity',
                name: 'selected_entity',
                polyline: {
                    positions: destination,
                    material: Cesium.Color.GREEN.withAlpha(0.7)
                }
            });
            if (entity) {
                viewer.flyTo(entity, {
                    offset: {
                        heading: Cesium.Math.toRadians(0.0),
                        pitch: Cesium.Math.toRadians(-90),
                        range: 0
                    }
                });
            }
        })
    } else if (geomtype.indexOf("point") > -1) {
        var points = [];
        addProjectionTransform(geom.coordinates, inSR, outSR, true, false, function (e) {
            let coordinates = e.result
            destination = Cesium.Cartesian3.fromDegrees(coordinates[0], coordinates[1]);
            entity = viewer.entities.add({
                id: 'selected_entity',
                name: 'selected_entity',
                position: destination,
                point: {
                    color: Cesium.Color.GREEN.withAlpha(0.7),
                    pixelSize: 10,
                    heightReference: Cesium.HeightReference.CLAMP_TO_GROUND,
                    outlineColor: Cesium.Color.YELLOW,
                    outlineWidth: 2,
                }
            });
            if (entity) {
                viewer.flyTo(entity, {
                    offset: {
                        heading: Cesium.Math.toRadians(0.0),
                        pitch: Cesium.Math.toRadians(-90),
                        range: 0
                    }
                });
            }
        })
    }
}

/**
 * 移除xt3d简单标注
 */
export function removeSampleLabel() {
    if (sampleLabel) {
        sampleLabel.remove()
        sampleLabel = null;
    }
}
