/*
 * @Descripttion: 
 * @version: 
 * @Author: 牛夏蕊
 * @Date: 2022-07-29 11:00:38
 * @LastEditors: 牛夏蕊
 * @LastEditTime: 2022-08-16 10:19:39
 */
import * as Cesium from "cesium";
import { removeSampleLabel } from "@/utils/map/fun/layerQuery";
import { GlobeTooltip } from "@/utils/map/drawPlugin/GlobeTooltip";
import { clearTrack } from "@/utils/map/fun/layerDraw";
import { ClearOrbitFlightCamera } from "@/utils/map/fly/fly";

let g_tooltip = undefined;
export function setTooltip(viewer) {
    if (!g_tooltip) {
        g_tooltip = new GlobeTooltip(viewer.container);
    }
    return g_tooltip
}
let g_cesium_handler = undefined;
export function setScreenSpaceEventHandler(viewer) {
    if (!g_cesium_handler) {
        g_cesium_handler = new Cesium.ScreenSpaceEventHandler(viewer.scene.canvas);
    }
    return g_cesium_handler
}
export function clearScreenSpaceEventHandler() {
    if (g_cesium_handler) {
        g_cesium_handler.destroy();
        g_cesium_handler = undefined;
    }
}
let g_onTickEventRemoveCallback = undefined
export function controlOnTickEvent(func) {
    if (!g_onTickEventRemoveCallback && func) {
        g_onTickEventRemoveCallback = func
    } else if (g_onTickEventRemoveCallback) {
        // .onTick.addEventListener返回值是一个移除的回调函数，它可以清除onTickCallback事件
        g_onTickEventRemoveCallback();
        g_onTickEventRemoveCallback = undefined
    }
}

//将鼠标的二维坐标转换为对应椭球体三维坐标
export function getPickPositionGlobe(position, viewer) {
    var cartesian = null;
    if (viewer.scene.globe.depthTestAgainstTerrain) {
        //cartesian = viewer.camera.pickEllipsoid(position, viewer.scene.globe.ellipsoid);
        cartesian = viewer.scene.pickPosition(position);
    } else {
        var ray = viewer.camera.getPickRay(position);
        cartesian = viewer.scene.globe.pick(ray, viewer.scene);
    }
    return cartesian;
}
/**
 * 实时获取鼠标所在位置
 * @param {*} ele
 * @return {*}
 */
export function getMousePosition(ele, viewer) {
    ele.style.display = "block";
    //得到当前三维场景
    var scene = viewer.scene;
    //得到当前三维场景的椭球体
    var ellipsoid = scene.globe.ellipsoid;

    var longitudeString = null;
    var latitudeString = null;
    var elevationHeight = null;
    var height = null;
    var cartesian = null;
    // 定义当前场景的画布元素的事件处理
    var handler = new Cesium.ScreenSpaceEventHandler(scene.canvas);
    //设置鼠标移动事件的处理函数，这里负责监听x,y坐标值变化
    handler.setInputAction(function (movement) {
        //通过指定的椭球或者地图对应的坐标系，将鼠标的二维坐标转换为对应椭球体三维坐标
        //cartesian = viewer.camera.pickEllipsoid(movement.endPosition, ellipsoid);
        cartesian = getPickPositionGlobe(movement.endPosition, viewer);
        if (cartesian) {
            //将笛卡尔坐标转换为地理坐标
            var cartographic =
                ellipsoid.cartesianToCartographic(cartesian);
            //将弧度转为度的十进制度表示
            longitudeString = Cesium.Math.toDegrees(
                cartographic.longitude
            );
            latitudeString = Cesium.Math.toDegrees(
                cartographic.latitude
            );
            //获取相机高度
            height = Math.ceil(
                viewer.camera.positionCartographic.height
            );
            //海拔高度
            elevationHeight = viewer.scene.globe.getHeight(cartographic)
                ? viewer.scene.globe.getHeight(cartographic).toFixed(2)
                : 0;

            longitudeString = longitudeString.toFixed(6);
            latitudeString = latitudeString.toFixed(6);
            ele.innerHTML = `经度:${longitudeString}, 纬度:${latitudeString},视角高度:${height}米,海拔高度:${elevationHeight}米`;
        }
    }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);

    //设置鼠标滚动事件的处理函数，这里负责监听高度值变化
    handler.setInputAction(function (wheelment) {
        height = Math.ceil(viewer.camera.positionCartographic.height);
        ele.innerHTML = `经度:${longitudeString}, 纬度:${latitudeString},视角高度:${height}米,海拔高度:${elevationHeight}米`;
    }, Cesium.ScreenSpaceEventType.WHEEL);
}
/**
 * 隐藏鼠标位置
 * @param {*} ele
 * @return {*}
 */
export function removeMousePosition(ele) {
    ele.style.display = "none";
}
/**
 * Cesium缩放最低高度设置
 * @return {*}
 */
export function forbidDown(viewer) {
    var scene = viewer.scene;
    var canvas = viewer.canvas;
    var camera = viewer.camera;
    scene.screenSpaceCameraController.minimumZoomDistance = 0; //距离地形的距离？这个值可以多测试几个值，，我这不太好描述
    viewer.clock.onTick.addEventListener(function () {
        setMinCamera();
    });
    var setMinCamera = function () {
        if (camera.pitch > 0) {
            scene.screenSpaceCameraController.enableTilt = false;
        }
    };
    var startMousePosition;
    var mousePosition;
    var handler = new Cesium.ScreenSpaceEventHandler(canvas);
    handler.setInputAction(function (movement) {
        mousePosition = startMousePosition = Cesium.Cartesian3.clone(
            movement.position
        );
        handler.setInputAction(function (movement) {
            mousePosition = movement.endPosition;
            var y = mousePosition.y - startMousePosition.y;
            if (y > 0) {
                scene.screenSpaceCameraController.enableTilt = true;
            }
        }, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
    }, Cesium.ScreenSpaceEventType.MIDDLE_DOWN);
}
export function RemoveAllLayers(viewer) {
    // 清空所有实体
    removeAllEntities(viewer);
    // 移除点选、框选鼠标监听
    // identifyClear();
    clearScreenSpaceEventHandler();
    // 移除xt3d简单标注
    removeSampleLabel();
    // 移除绑定在viewer实例上Cesium鼠标事件
    removeMouseAction(viewer)
    // 清除绘制
    clearTrack(viewer)
    // 清除绕点飞行监听
    ClearOrbitFlightCamera(viewer)
    // 移除雨雪雾
    viewer.scene.postProcessStages.removeAll();
    // 恢复地形夸张
    if (viewer.scene.terrainExaggeration != 1) {
        viewer.scene.terrainExaggeration = 1;
        viewer.scene.globe.terrainExaggeration = 1;
    }
    // 恢复场景颜色控制
    restorColorSetting(viewer);
    // 清空onTick监听
    controlOnTickEvent();
    // 清除等高线
    viewer.scene.globe.material = undefined;
}
// 清空所有实体
export function removeAllEntities(viewer) {
    for (var i = 0; i < viewer.entities.values.length;) {
        var entityId = viewer.entities.values[i].id;
        if (entityId && (entityId.indexOf("video_entity_") > -1 || entityId.indexOf("story_") > -1)) {
            i++;
        } else {
            viewer.entities.remove(viewer.entities.values[i]);
        }
    }
}
// 一键移除Cesium鼠标事件（在viewer实例上）
export function removeMouseAction(viewer) {
    viewer.screenSpaceEventHandler.removeInputAction(
        Cesium.ScreenSpaceEventType.LEFT_CLICK
    );
    viewer.screenSpaceEventHandler.removeInputAction(
        Cesium.ScreenSpaceEventType.LEFT_DOUBLE_CLICK
    );
    viewer.screenSpaceEventHandler.removeInputAction(
        Cesium.ScreenSpaceEventType.MOUSE_MOVE
    );
    viewer.screenSpaceEventHandler.removeInputAction(
        Cesium.ScreenSpaceEventType.RIGHT_CLICK
    );
}
//获取视图范围
export function GetViewExtent(viewer) {
    var extent = {};
    var scene = viewer.scene;
    var ellipsoid = scene.globe.ellipsoid;
    var canvas = scene.canvas;

    var car3_lt = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(0, 0), ellipsoid); // canvas左上角
    var car3_rb = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(canvas.width, canvas.height), ellipsoid); // canvas右下角

    // 当canvas左上角和右下角全部在椭球体上
    if (car3_lt && car3_rb) {
        var carto_lt = ellipsoid.cartesianToCartographic(car3_lt);
        var carto_rb = ellipsoid.cartesianToCartographic(car3_rb);
        extent.xmin = Cesium.Math.toDegrees(carto_lt.longitude);
        extent.ymax = Cesium.Math.toDegrees(carto_lt.latitude);
        extent.xmax = Cesium.Math.toDegrees(carto_rb.longitude);
        extent.ymin = Cesium.Math.toDegrees(carto_rb.latitude);
    } else if (!car3_lt && car3_rb) { // 当canvas左上角不在但右下角在椭球体上
        var car3_lt2 = null;
        var yIndex = 0;
        var xIndex = 0;
        do {
            // 这里每次10像素递加，一是10像素相差不大，二是为了提高程序运行效率
            yIndex <= canvas.height ? yIndex += 10 : canvas.height;
            xIndex <= canvas.width ? xIndex += 10 : canvas.width;
            car3_lt2 = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(xIndex, yIndex), ellipsoid);
        } while (!car3_lt2);
        var carto_lt2 = ellipsoid.cartesianToCartographic(car3_lt2);
        var carto_rb2 = ellipsoid.cartesianToCartographic(car3_rb);
        extent.xmin = Cesium.Math.toDegrees(carto_lt2.longitude);
        extent.ymax = Cesium.Math.toDegrees(carto_lt2.latitude);
        extent.xmax = Cesium.Math.toDegrees(carto_rb2.longitude);
        extent.ymin = Cesium.Math.toDegrees(carto_rb2.latitude);
    } else if (car3_lt && !car3_rb) { // 当canvas左上角在但右下角不在椭球体上
        var car3_rb2 = null;
        var yIndex = canvas.height;
        var xIndex = canvas.width;
        do {
            // 这里每次10像素递加，一是10像素相差不大，二是为了提高程序运行效率
            yIndex >= 10 ? yIndex -= 10 : 10;
            xIndex >= 10 ? xIndex -= 10 : 10;
            car3_rb2 = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(yIndex, yIndex), ellipsoid);
        } while (!car3_rb2);
        var carto_lt2 = ellipsoid.cartesianToCartographic(car3_lt);
        var carto_rb2 = ellipsoid.cartesianToCartographic(car3_rb2);
        extent.xmin = Cesium.Math.toDegrees(carto_lt2.longitude);
        extent.ymax = Cesium.Math.toDegrees(carto_lt2.latitude);
        extent.xmax = Cesium.Math.toDegrees(carto_rb2.longitude);
        extent.ymin = Cesium.Math.toDegrees(carto_rb2.latitude);
    } else if (!car3_lt && !car3_rb) {
        var car3_lt2 = null;
        var yIndex = 0;
        var xIndex = 0;
        do {
            // 这里每次10像素递加，一是10像素相差不大，二是为了提高程序运行效率
            yIndex <= canvas.height ? yIndex += 10 : canvas.height;
            xIndex <= canvas.width ? xIndex += 10 : canvas.width;
            car3_lt2 = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(xIndex, yIndex), ellipsoid);
        } while (!car3_lt2);

        var car3_rb2 = null;
        var yIndex = canvas.height;
        var xIndex = canvas.width;
        do {
            // 这里每次10像素递加，一是10像素相差不大，二是为了提高程序运行效率
            yIndex >= 10 ? yIndex -= 10 : 10;
            xIndex >= 10 ? xIndex -= 10 : 10;
            car3_rb2 = viewer.camera.pickEllipsoid(new Cesium.Cartesian2(yIndex, yIndex), ellipsoid);
        } while (!car3_rb2);

        var carto_lt2 = ellipsoid.cartesianToCartographic(car3_lt2);
        var carto_rb2 = ellipsoid.cartesianToCartographic(car3_rb2);
        extent.xmin = Cesium.Math.toDegrees(carto_lt2.longitude);
        extent.ymax = Cesium.Math.toDegrees(carto_lt2.latitude);
        extent.xmax = Cesium.Math.toDegrees(carto_rb2.longitude);
        extent.ymin = Cesium.Math.toDegrees(carto_rb2.latitude);
    }

    // 获取高度
    //extent.height = Math.ceil(viewer.camera.positionCartographic.height);
    return [extent.xmin, extent.ymin, extent.xmax, extent.ymax];
}

/**
 * @Desc: 将数据转换为Geojson格式
 * @Author: 牛夏蕊
 * @Date: 2022-08-01 17:16:24
 * @param {*} data 源数据
 * @param {*} type 源数据类型 arcgis
 * @return {*}
 */
export function handleToGeojson(data, type) {
    var geojson = {
        "type": "FeatureCollection",
        "features": []
    };
    if (!data) {
        return geojson;
    }
    type = type ? type : "arcgis";
    switch (type) {
        case "arcgis":
            for (var i = 0; i < data.length; i++) {
                var param = data[i];
                var arcgis = {
                    displayFieldName: "",
                    fieldAliases: {},
                    fields: [],
                    features: [param],
                    geometryType: param.geometryType,
                    spatialReference: param.geometry.spatialReference
                }
                if (param.geometryType == "esriGeometryPolygon" || param.geometry.rings) {
                    geojson.features.push({
                        "type": "Feature",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [param.geometry.rings]
                        },
                        "properties": param.attributes,
                        "arcgis": arcgis
                    });
                } else if (param.geometryType == "esriGeometryPolyline" || param.geometry.path || param.geometry.paths) {
                    geojson.features.push({
                        "type": "Feature",
                        "geometry": {
                            "type": "Polyline",
                            "coordinates": [param.geometry.path || param.geometry.paths]
                        },
                        "properties": param.attributes,
                        "arcgis": arcgis
                    });
                } else if (param.geometryType == "esriGeometryPoint" || param.geometry.x) {
                    geojson.features.push({
                        "type": "Feature",
                        "geometry": {
                            "type": "Point",
                            "coordinates": [param.geometry.x, param.geometry.y]
                        },
                        "properties": param.attributes,
                        "arcgis": arcgis
                    });
                }
            }
            break;
        default:
            break;
    }
    return geojson;
}

/**
 * 采集高度
 * @param {*} cartesians：笛卡尔坐标组成的数组
 * @param {*} type：
 * @returns 采集结果：笛卡尔坐标组成的数组
 */
export function getHeightByType(positions, type) {
    if (type === 'model') {
        return new Promise(async resolve => {
            try {
                let promise = viewer.scene.clampToHeightMostDetailed(cartesians)
                promise.then(updatedCartesians => resolve(updatedCartesians))
            } catch (e) {
                resolve(false)
            }
        })
    } else if (type = 'terrain') {
        let terrain = viewer.terrainProvider
        return new Promise(async resolve => {
            try {
                // 当前场景中没有使用地形
                if (!terrain) resolve(false)
                let promise = Cesium.sampleTerrain(terrain, 15, positions);
                promise.then(function (updatedPositions) {
                    resolve(updatedPositions)
                });
            } catch (e) {
                resolve(false)
            }
        })
    }
}
/**
 * 根据四至坐标进行视图缩放
 * @param minx
 * @param miny
 * @param maxx
 * @param maxy
 */
export function flyByRectangle(params) {
    // var minx = params["MINX"];
    // var miny = params["MINY"];
    // var maxx = params["MAXX"];
    // var maxy = params["MAXY"];
    // if(minx && miny && maxx && maxy){
    //     var rectangle = new Cesium.Rectangle.fromDegrees(minx, miny, maxx, maxy);
    //     viewer.camera.flyTo({
    //         destination: rectangle,
    //         duration: 3,
    //         //获取canvas截图
    //         // complete: function (e){
    //         //     var canvas = viewer.scene.canvas;
    //         //     var mapURL = canvas.toDataURL("image/png");
    //         //     overlayVectorFeatureDataURL[params.name] = mapURL.substring(22);
    //         // }
    //     });
    // }

    //2022/3/28 迁移鄂尔多斯方法
    var minx = params["MINX"];
    var miny = params["MINY"];
    var maxx = params["MAXX"];
    var maxy = params["MAXY"];
    var isLoadTerrain = false
    //判断一下是否已加载了地形数据
    for (var attr in cesium.layer3DList) {
        var layerItem = cesium.layer3DList[attr];
        if (layerItem["NAME"] && layerItem["SERVICETYPE"] == sw_servicetype.sw_terrain && layerItem["SOURCE"] == sw_source.sw_mongodb) {
            isLoadTerrain = true
        }
    }

    if (minx && miny && maxx && maxy) {
        //如果已经加载了地形数据，需要通过terrainProvider获取高程数据
        //取出面的8个点坐标，拿点坐标去求高度值
        var slopelineposition = []
        slopelineposition.push(Cesium.Cartographic.fromDegrees(minx, miny));
        slopelineposition.push(Cesium.Cartographic.fromDegrees(maxx, maxy));
        slopelineposition.push(Cesium.Cartographic.fromDegrees(minx, maxy));
        slopelineposition.push(Cesium.Cartographic.fromDegrees(maxx, miny));
        if (true) {
            var terrainProvider = viewer.terrainProvider;
            var promise = Cesium.sampleTerrain(terrainProvider, 15, slopelineposition);
            Cesium.when(promise,
                function (updatedPositions) {
                    var rect = Cesium.Rectangle.fromCartesianArray(Cesium.Cartesian3.fromDegreesArrayHeights(
                        [minx, miny, updatedPositions[0].height,
                            maxx, maxy, updatedPositions[1].height,
                            minx, maxy, updatedPositions[2].height,
                            maxx, miny, updatedPositions[3].height
                        ]))
                    loactionTectEntity = viewer.entities.add({
                        name: 'locationRectangle',
                        // id: 'locationRectangle',
                        rectangle: {
                            coordinates: rect,
                            material: Cesium.Color.BLUE.withAlpha(0),
                            height: updatedPositions[0].height,
                            outline: false
                        }
                    });
                    var flyPromise = viewer.flyTo(loactionTectEntity, {
                        duration: 3,
                        offset: new Cesium.HeadingPitchRange(0.0, Cesium.Math.toRadians(-90.0))
                    });

                    flyPromise.then(function () {
                        setPitch();
                    });

                    function setPitch() {
                        var center = Cesium.Rectangle.center(rect);
                        var car = Cesium.Cartesian3.fromRadians(center.longitude, center.latitude);
                        var range = Cesium.Cartesian3.distance(car, viewer.camera.position) * Math.cos(20);
                    }
                })
        } else {
            var cartesian3List = [];
            var cartesian3 = Cesium.Cartesian3.fromDegrees(minx, miny, 500000);
            cartesian3List.push(cartesian3);
            var cartesian32 = Cesium.Cartesian3.fromDegrees(maxx, maxy, 500000);
            cartesian3List.push(cartesian32);
            var rectangle = Cesium.Rectangle.fromCartesianArray(cartesian3List);
            viewer.camera.flyTo({
                destination: rectangle,
                duration: 3
            });
        }
    }
}

/**
 * 返回uuid
 */
export function getUUID() {
    var s = [];
    var hexDigits = "0123456789abcdef";
    for (var i = 0; i < 36; i++) {
        s[i] = hexDigits.substr(Math.floor(Math.random() * 0x10), 1);
    }
    s[14] = "4"; // bits 12-15 of the time_hi_and_version field to 0010
    s[19] = hexDigits.substr((s[19] & 0x3) | 0x8, 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
    s[8] = s[13] = s[18] = s[23] = "-";

    var uuid = s.join("");
    return uuid;
}
// 恢复场景颜色设置
export function restorColorSetting(viewer) {
    for (
        let i = 1;
        i < viewer.imageryLayers.length;
        i++
    ) {
        let layer = viewer.imageryLayers.get(i);
        if (layer) {
            layer.brightness = 1;
            layer.contrast = 1;
            layer.hue = 0;
            layer.saturation = 1;
            layer.gamma = 1;
        }
    }
}
//镜头拉近 放大
export function zoomInViewer(viewer) {
    // 获取当前镜头位置的笛卡尔坐标
    let cameraPos = viewer.camera.position;
    // 获取当前坐标系标准
    let ellipsoid = viewer.scene.globe.ellipsoid;
    // 根据坐标系标准，将笛卡尔坐标转换为地理坐标
    let cartographic = ellipsoid.cartesianToCartographic(cameraPos);
    // 获取镜头的高度
    let height = cartographic.height;
    // 根据上面当前镜头的位置，获取该中心位置的经纬度坐标
    let centerLon = parseFloat(Cesium.Math.toDegrees(cartographic.longitude).toFixed(8));
    let centerLat = parseFloat(Cesium.Math.toDegrees(cartographic.latitude).toFixed(8));
    viewer.camera.flyTo({
        destination: Cesium.Cartesian3.fromDegrees(centerLon, centerLat, height / 1.8),
        duration: 1.0,
        orientation: {
            heading: viewer.camera.heading,
            pitch: viewer.camera.pitch,
            roll: viewer.camera.roll
        }
    });
}

//镜头拉远 缩小
export function zoomOutViewer(viewer) {
    // 获取当前镜头位置的笛卡尔坐标
    let cameraPos = viewer.camera.position;
    // 获取当前坐标系标准
    let ellipsoid = viewer.scene.globe.ellipsoid;
    // 根据坐标系标准，将笛卡尔坐标转换为地理坐标
    let cartographic = ellipsoid.cartesianToCartographic(cameraPos);
    // 获取镜头的高度
    let height = cartographic.height;
    // 根据上面当前镜头的位置，获取该中心位置的经纬度坐标
    let centerLon = parseFloat(Cesium.Math.toDegrees(cartographic.longitude).toFixed(8));
    let centerLat = parseFloat(Cesium.Math.toDegrees(cartographic.latitude).toFixed(8));
    viewer.camera.flyTo({
        destination: Cesium.Cartesian3.fromDegrees(centerLon, centerLat, height * 1.8),
        duration: 1.0,
        orientation: {
            heading: viewer.camera.heading,
            pitch: viewer.camera.pitch,
            roll: viewer.camera.roll
        }
    });
}
/**
 * 设置罗盘显示及位置
 * @param params
 *      top  number
 *      left number
 *      size number
 */
export function setCompassPosition(viewer, params) {
    if (typeof (params) == "string") {
        params = JSON.parse(params);
    }
    var options = {};
    options.enableCompass = true; //罗盘
    options.enableZoomControls = false; //缩放
    options.enableDistanceLegend = false; //比例尺
    options.enableCompassOuterRing = true; //指南针外环
    viewer.extend(Cesium.viewerCesiumNavigationMixin, options);
    var left = parseFloat(params.left);
    var top = parseFloat(params.top);
    var size = parseFloat(params.size);
    $(".navigation-controls").hide();
    $(".compass").animate({
        top: top + "px",
        left: left + "px"
    });
    if (size) {
        //调整主面板大小
        $(".compass, .compass-outer-ring").css("width", size + "px");//主单元块
        $(".compass, .compass-outer-ring").css("fill", "#156BC0");//主单元块
        $(".compass, .compass-outer-ring").css("height", size + "px");//主单元块

        $(".compass").css("line-height", size + "px");
        $(".compass").css("text-align", "center");

        $(".compass-gyro").css("width", size + "px") //主单元内嵌块
        $(".compass-gyro").css("height", size + "px") //主单元内嵌块

        $(".compass-rotation-marker").css("width", size + "px") //主单元内嵌块
        $(".compass-rotation-marker").css("height", size + "px") //主单元内嵌块

        $(".compass-outer-ring-background").css("position", "relative") // 蓝色背景圈
        $(".compass-outer-ring-background").css("display", "none") // 蓝色背景圈
        $(".compass-outer-ring-background").css("width", (44 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-outer-ring-background").css("height", (44 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-outer-ring-background").css("left", "0px") // 蓝色背景圈
        $(".compass-outer-ring-background").css("top", (14 * (size / 44) / 2 + 5) + "px") // 蓝色背景圈


        $(".compass-gyro-background").css("width", (33 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-gyro-background").css("height", (33 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-gyro-background").css("left", (30 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-gyro-background").css("top", (30 * (size / 95)) + "px") // 蓝色背景圈
        $(".compass-gyro-background").css("border-radius", "50px") // 蓝色背景圈
    }
}
