//lgx加载
import * as Cesium from "cesium";
import mapRequest from "@/utils/map/mapRequest";
import { getHeightByType } from "@/utils/map/mapUtils";

import { checkLayerTransparent } from "@/utils/map/layerHelper"

export function LoadJsonServer(layerInfo) {
    var LoadJsonServer = new Object;
    LoadJsonServer.id = layerInfo.hasOwnProperty("GUID") ? layerInfo.GUID : "";
    LoadJsonServer.style = layerInfo.hasOwnProperty("STYLE") ? layerInfo.STYLE : "#0033FF";
    LoadJsonServer.lineWidth = layerInfo.hasOwnProperty("WIDTH") ? layerInfo.WIDTH : 5;
    LoadJsonServer.speed = layerInfo.hasOwnProperty("SPEED") ? layerInfo.SPEED : 1;
    LoadJsonServer.pointColor = layerInfo.hasOwnProperty("pointColor") ? layerInfo.pointColor : "#FFFFFF";
    LoadJsonServer.url = layerInfo.hasOwnProperty("URL") ? layerInfo.URL : "";
    LoadJsonServer.name = layerInfo.hasOwnProperty("NAME") ? layerInfo.NAME : "";
    LoadJsonServer.collection = new Cesium.PrimitiveCollection();
    LoadJsonServer.hotSpotBoards = [];
    LoadJsonServer.entitycollection = new Cesium.CustomDataSource("clickEntityCollection");

    LoadJsonServer.SetVisible = function (visible) {
        this.collection.show = visible;
        for (var i = 0; i < this.hotSpotBoards.length; i++) {
            this.hotSpotBoards[i].setVisible(visible);
        }
        if (this.entitycollection) {
            this.entitycollection.show = visible;
        }
    }
    LoadJsonServer.CreateLayer = function (viewer) {
        let url = this.url;
        let labels = new Cesium.LabelCollection();
        let billboards = new Cesium.BillboardCollection();
        // let linescollection = new Cesium.PolylineCollection();
        let collection = this.collection;
        let entitycollection = this.entitycollection;
        collection.add(labels);
        collection.add(billboards);
        viewer.dataSources.add(entitycollection);
        viewer.scene.primitives.add(collection);
        viewer.scene.globe.depthTestAgainstTerrain = false;
        let position;
        this.hotSpotBoards = [];
        let data = null, geojson = null, Global = null, Attachment = null;
        let allPoints = [];
        let that = this;
        mapRequest({
            url: url,
            method: 'get',
        }).then((res) => {
            data = typeof res == "string" ? JSON.parse(res) : res

            geojson = data.Features;//geojson
            Global = data.Global;//全局变量
            Attachment = data.Attachment;//资源信息
            viewer.screenSpaceEventHandler.setInputAction(function onLeftClick(movement) {
                let pickedFeature = viewer.scene.pick(movement.position);
                if (typeof (pickedFeature) != "undefined") {
                    let source = Attachment[pickedFeature.id._id];
                    let Container = Global.Symbol.Container;
                    let Position = Global.Symbol.Position;
                    let obj = {
                        "Container": Container,
                        "Position": Position,
                        "Name": pickedFeature.id.name,
                    }
                    renderHotPointInfoForm(source, obj);
                    // alert("你点击了" + pickedFeature.id._id);
                }
            }, Cesium.ScreenSpaceEventType.LEFT_CLICK);

            for (let p = 0; p < geojson.features.length; p++) {
                allPoints.push(Cesium.Cartographic.fromDegrees(geojson.features[p].geometry.coordinates[0], geojson.features[p].geometry.coordinates[1]));
            }
            getHeightByType(allPoints, 'terrain').then(results => {
                for (let k = 0; k < results.length; k++) {
                    position = Cesium.Cartesian3.fromDegrees(geojson.features[k].geometry.coordinates[0], geojson.features[k].geometry.coordinates[1], (results[k]["height"] || 0) + Global.Symbol.PositionHeight);
                    addLabel(geojson.features[k], position, Global.Symbol);
                }
            });
            window.treeList[id].imageryLayer = that
        })

        /**
         *加载标签
         */
        function addLabel(features, position, Symbol) {
            entitycollection.entities.add({
                position: position,
                name: features.properties.NAME,
                id: features.properties.ID,
                label: { //文字标签
                    text: features.properties.NAME,
                    scale: Symbol.Scale,
                    font: Symbol.FontSize,
                    scaleByDistance: new Cesium.NearFarScalar(Symbol.Level / 10, 1, Symbol.Level, 0.6),
                    style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                    fillColor: Cesium.Color.WHITE,
                    horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
                    verticalOrigin: Cesium.VerticalOrigin.CENTER,
                    distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, Symbol.Level),
                    //heightReference: Cesium.HeightReference.RELATIVE_TO_GROUND, //CLAMP_TO_GROUND//RELATIVE_TO_GROUND
                    outlineWidth: 1,
                    outlineColor: Cesium.Color.WHITE,
                    pixelOffset: new Cesium.Cartesian2(15, -2)//偏移量
                },
                billboard: {
                    image: '/MapServer/Portal/img/mark3.png',
                    show: true,
                    width: Symbol.FontSize * 0.73,
                    height: Symbol.FontSize,
                    horizontalOrigin: Cesium.HorizontalOrigin.CENTER,
                    verticalOrigin: Cesium.VerticalOrigin.CENTER,
                    scale: Symbol.Scale,
                    // distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, Symbol.Level),
                }
            });
        }
        // cesium.layer3DList[that.id] = that;
    }
    return LoadJsonServer;
}
/**
 * 渲染热点详情弹窗
 * 视频、图片、文本在TAB也内轮播展示
 * @param {*} obj 
 */
var HotPointInfoDOM
function renderHotPointInfoForm(obj, domParam) {
    HotPointInfoDOM && HotPointInfoDOM.remove();
    var containerStr = domParam.Container
    var container = containerStr.split(",")
    var domWidth = container[2] + "px";
    var domHeight = container[3] + "px";
    var domTitle = domParam.Name;
    if (domParam.Name.length > 20) {
        domTitle = domParam.Name.substring(0, 20) + "..."
    }
    var domCarouselHeight = (Number(container[3]) - 40).toString() + "px"
    if (typeof (obj) == "object" && Object.keys(obj).length > 0) {
        var datas = Object.entries(obj);
        var carouselKeys = [];
        var tabTitleHtml = "",
            tabContentHtml = "";
        var indexTAG = 0
        datas.map(function (item, index) {
            var key = item[0];
            var value = item[1];
            if (value.length > 0) {
                var tabTitle = "";
                var carouselContent = "";
                var carouselHTML = "";
                switch (key) {
                    case "Video":
                        tabTitle = "视频";
                        if (value.length > 0) {
                            carouselKeys.push("carousel_" + key)
                            for (var i = 0; i < value.length; i++) {
                                var videoName = value[i].Name
                                var videoUrl = value[i].Url
                                carouselContent += `<video id="myVideo" controls autoplay loop muted style="width: 100%;height: 100%;">
                                                     <source src="` + videoUrl + `" type="video/mp4">
                                                 </video>`
                            }
                        } else {
                            carouselContent = ""
                        }
                        break;
                    case "Image":
                        tabTitle = "图片";
                        if (value.length > 0) {
                            carouselKeys.push("carousel_" + key)
                            for (var i = 0; i < value.length; i++) {
                                var ImageName = value[i].Name
                                var ImageUrl = value[i].Url
                                carouselContent += `<div style="width:100%;height:100%;position:relative">
                         <div style="display:none;position: absolute;z-index: 99;width: 100%;top: 10px;left: 0;text-align: center;font-size: 16px;
                         font-weight: bold;padding: 0 20px;letter-spacing: 2px;color:#1096f1;-webkit-text-stroke: 0.5px #FFF;">` + ImageName + `</div>
                         <img style="position: absolute;width:100%;height:100%;" src="` + ImageUrl + `" style="width:100%;height:100%;"></div>`
                            }
                        } else {
                            carouselContent = ""
                        }
                        break;
                    case "Text":
                        tabTitle = "文本";
                        if (value.length > 0) {
                            carouselKeys.push("carousel_" + key)
                            for (var i = 0; i < value.length; i++) {
                                var txtName = value[i].Name;
                                var txtTitle = value[i].Title;
                                var txtContent = value[i].Content
                                carouselContent += `<div style="background-color: #00c4dc;">
                                                     <div class="txtBoxTitle">${txtTitle}</div>
                                                     <div class="txtBoxContent">${txtContent}</div>
                                                 </div>`
                            }
                        } else {
                            carouselContent = ""
                        }
                        break;
                    default:
                        break;
                }
                if (carouselContent != "") {
                    carouselHTML = `<div class="layui-carousel" id="carousel_${key}" style="padding:10px;height:100%;background-color: #00c4dc;">
                                     <div carousel-item>${carouselContent}</div>
                                 </div>`
                } else {
                    carouselHTML = `<div style="height: 100%;display: flex;align-items: center;justify-content: center;font-size: 20px;">暂无内容!</div>`
                }
                if (indexTAG == 0) {
                    tabTitleHtml += `<li class="layui-this">${tabTitle}</li>`
                    tabContentHtml += `<div class="layui-tab-item layui-show" style="height:100%">${carouselHTML}</div>`
                } else {
                    tabTitleHtml += `<li>${tabTitle}</li>`
                    tabContentHtml += `<div class="layui-tab-item" style="height:100%">${carouselHTML}</div>`
                }
                indexTAG++
            }
        })
        var formDIV = document.createElement("div");
        formDIV.className = ""
        formDIV.innerHTML = `<div style="position: absolute;width:100%;height:40px;display: flex;align-items: center;justify-content: space-between;padding: 0 10px;">
                                 <div title="${domParam.Name}" style="color: #FFF;font-size: 18px;font-weight: bold;margin-left: 10px;">${domTitle}</div>
                                 <img id="closeHotPointInfoDOM" style="cursor: pointer;" title="关闭" src="/MapServer/Portal/js/fun/fly/images/circle_close.png">
                             </div>
                             <div class="layui-tab layui-tab-brief" style="margin: 0;" lay-filter="">
                                 <ul id="TitleBarHotPointInfo" class="layui-tab-title" style="text-align: right;margin: 0 40px;border:none">${tabTitleHtml}</ul>
                                 <div class="layui-tab-content" style="height:${domCarouselHeight};padding-bottom: 0;border-top: 1px solid;">${tabContentHtml}</div>
                             </div>`

        formDIV.style.width = domWidth;
        formDIV.style.height = domHeight;
        formDIV.className = "form_hotpoint_info"
        formDIV.style.position = "absolute";
        var domPosition = domParam.Position
        switch (domPosition) {
            case "BottomLeft":
                formDIV.style.bottom = container[0] + "px"
                formDIV.style.left = container[1] + "px"
                break;
            case "BottomRight":
                formDIV.style.bottom = container[0] + "px"
                formDIV.style.right = container[1] + "px"
                break;
            case "TopLeft":
                formDIV.style.top = container[0] + "px"
                formDIV.style.left = container[1] + "px"
                break;
            case "TopRight":
                formDIV.style.top = container[0] + "px"
                formDIV.style.right = container[1] + "px"
                break;
            default:
                break
        }
        HotPointInfoDOM = formDIV
        document.body.appendChild(HotPointInfoDOM);
        if (!layui.carousel) {
            layui.use(['carousel'], function () {
                renderCarouse(carouselKeys)
            });
        } else {
            renderCarouse(carouselKeys)
        }
        $("#closeHotPointInfoDOM").click(function (e) {
            HotPointInfoDOM.remove();
        })
        $('#TitleBarHotPointInfo').mousedown(function () {
            toolbarMove(this.parentNode);
        });
    } else {
        console.log("参数传递错误")
        layer.msg("暂无配置信息！")
    }
}
/**
 * 批量初始化渲染layui轮播
 * @param {*} domIds
 */
function renderCarouse(domIds) {
    if (domIds.length > 0) {
        domIds.map(function (item) {
            layui.carousel.render({
                elem: '#' + item,
                width: '100%', //设置容器宽度
                height: '100%',
                arrow: 'hover', //始终显示箭头
                autoplay: true,
                interval: 2000,
                anim: 'fade', //切换动画方式，可选fade,default,updown
                indicator: 'inside' //指示器位置，可选inside,outside,none
            });
        })
    }
}