//lgx加载
import * as Cesium from "cesium";
import mapRequest from "@/utils/map/mapRequest";
import { getHeightByType } from "@/utils/map/mapUtils";

import { checkLayerTransparent } from "@/utils/map/layerHelper"

export function OpenJsonService(layerInfo) {
    var OpenJsonService = new Object;
    OpenJsonService.id = layerInfo.hasOwnProperty("GUID") ? layerInfo.GUID : "";
    OpenJsonService.style = layerInfo.hasOwnProperty("STYLE") ? layerInfo.STYLE : "#0033FF";
    OpenJsonService.lineWidth = layerInfo.hasOwnProperty("WIDTH") ? layerInfo.WIDTH : 5;
    OpenJsonService.speed = layerInfo.hasOwnProperty("SPEED") ? layerInfo.SPEED : 1;
    OpenJsonService.pointColor = layerInfo.hasOwnProperty("pointColor") ? layerInfo.pointColor : "#FFFFFF";
    OpenJsonService.url = layerInfo.hasOwnProperty("URL") ? layerInfo.URL : "";
    OpenJsonService.name = layerInfo.hasOwnProperty("NAME") ? layerInfo.NAME : "";
    OpenJsonService.collection = new Cesium.PrimitiveCollection();
    OpenJsonService.hotSpotBoards = [];
    OpenJsonService.entitycollection = new Cesium.CustomDataSource("clickEntityCollection");

    OpenJsonService.SetVisible = function (visible) {
        this.collection.show = visible;
        for (var i = 0; i < this.hotSpotBoards.length; i++) {
            this.hotSpotBoards[i].setVisible(visible);
        }
        if (this.entitycollection) {
            this.entitycollection.show = visible;
        }
    }
    OpenJsonService.CreateLayer = function (viewer) {
        let url = this.url;
        let numberindex = 0;
        let labels = new Cesium.LabelCollection();
        let linescollection = new Cesium.PolylineCollection();
        let collection = this.collection;
        let entitycollection = this.entitycollection;
        collection.add(labels);
        viewer.dataSources.add(entitycollection);
        viewer.scene.primitives.add(collection);
        viewer.scene.globe.depthTestAgainstTerrain = false;
        let position;
        this.hotSpotBoards = [];
        // let data = InitialParameter(url);
        let data = null;
        let that = this;

        mapRequest({
            url: url,
            method: 'get',
        }).then((res) => {
            data = typeof res == "string" ? JSON.parse(res) : res

            features = data.Entity;
            for (let i = 0; i < features.length; i++) {
                let GeometryType = features[i].GeometryType;
                let LabelType = features[i].Type;
                let Symbol = features[i].Symbol;
                let Geometry = features[i].Geometry.features;
                if (GeometryType == "Polygon") {
                    if (LabelType == "default") {
                        for (let j = 0; j < Geometry.length; j++) {
                            addPolygon(Geometry[j], Symbol);
                        }
                    }
                    if (LabelType == "custom") {
                        numberindex = 0;
                        for (let j = 0; j < Geometry.length; j++) {
                            addPolygonChange(Geometry[j], Symbol);
                        }
                    }
                }
                if (GeometryType == "Line") {
                    for (let l = 0; l < Geometry.length; l++) {
                        switch (LabelType) {
                            case "default":
                                addPolyline(Geometry[l], Symbol);
                                break;
                            case "custom":
                                addLineLighting(Geometry[l], Symbol);
                                break;
                            case "wakeLine":
                                wakeLine(Geometry[l], Symbol);
                                break;
                            default:
                                break;
                        }

                    }
                }
                if (GeometryType == "Point") {
                    let allPoints = [];
                    for (let p = 0; p < Geometry.length; p++) {
                        allPoints.push(Cesium.Cartographic.fromDegrees(Geometry[p].geometry.coordinates[0], Geometry[p].geometry.coordinates[1]));
                    }

                    if (LabelType == "default") {
                        getHeightByType(allPoints, 'terrain').then(results => {
                            for (let k = 0; k < results.length; k++) {
                                position = Cesium.Cartesian3.fromDegrees(Geometry[k].geometry.coordinates[0], Geometry[k].geometry.coordinates[1], (results[k]["height"] || 0));
                                addLabel(Geometry[k], position, Symbol);
                            }
                        });


                    }
                    if (LabelType == "custom") {
                        getHeightByType(allPoints, 'terrain').then(results => {
                            for (let k = 0; k < results.length; k++) {
                                position = Cesium.Cartesian3.fromDegrees(Geometry[k].geometry.coordinates[0], Geometry[k].geometry.coordinates[1], (results[k]["height"] || 0));
                                let hotSpotBoard = new CustomBoard(viewer, position, {
                                    chineseTitle: Geometry[k].properties.Text,
                                    englishTitle: "",
                                    bgImageUrl: "/MapServer/Portal/js/fun/fly/images/hotspotboard",
                                    level: 5000000
                                });
                                this.hotSpotBoards.push(hotSpotBoard);
                            }
                        });

                    }

                }
            }
            window.treeList[id].imageryLayer = that

        })

        /**
         *加载面
         */
        function addPolygon(feature, symbol) {
            let degreesArrayHeights = getDegreesArrayHeights(feature);
            let positions = Cesium.Cartesian3.fromDegreesArrayHeights(degreesArrayHeights);
            let polygon = new Cesium.PolygonGeometry({
                polygonHierarchy: new Cesium.PolygonHierarchy(positions),
                vertexFormat: Cesium.VertexFormat.ALL
            });
            let geometry = Cesium.PolygonGeometry.createGeometry(polygon);
            let instance = new Cesium.GeometryInstance({
                geometry: geometry
            });
            collection.add(new Cesium.Primitive({
                geometryInstances: instance,
                appearance: new xt3d.PolygonObject.PrimitiveGradientAppearance(Cesium.Color.fromCssColorString(symbol.color)),
            }));
        }

        /**
         *加载区域渐变面
         */
        function addPolygonChange(feature, symbol) {
            // number=Math.floor(Math.random()*symbol.color.length);
            let degreesArrayHeights = getDegreesArrayHeights(feature);
            let height = 1000 * feature.properties.Shape_Area;
            let polygon = new Cesium.PolygonGeometry({
                polygonHierarchy: new Cesium.PolygonHierarchy(
                    Cesium.Cartesian3.fromDegreesArrayHeights(degreesArrayHeights)),
                perPositionHeight: true,
                extrudedHeight: height,
                vertexFormat: Cesium.VertexFormat.ALL
            });
            let geometry = Cesium.PolygonGeometry.createGeometry(polygon);
            //primitive方式
            let instance = new Cesium.GeometryInstance({
                geometry: geometry
            });

            collection.add(new Cesium.Primitive({
                geometryInstances: instance,
                appearance: new xt3d.PolygonObject.PrimitiveGradientAppearance(new Cesium.Color.fromCssColorString(symbol.color[numberindex]).withAlpha(1)),
            }));
            numberindex++;
        }

        /**
         *获取坐标串
         */
        function getDegreesArrayHeights(feature) {
            let degreesArrayHeights = [];
            let coordinates;
            if (feature.geometry.type == "MultiPolygon") {
                coordinates = feature.geometry.coordinates[0][0];
            } else if (feature.geometry.type == "Polygon") {
                coordinates = feature.geometry.coordinates[0];
            }
            //坐标串转为需要的格式[x,y,z,x,y,z....]
            for (let i = 0; i < coordinates.length; i++) {
                const element = coordinates[i];
                degreesArrayHeights.push(element[0]);
                degreesArrayHeights.push(element[1]);
                degreesArrayHeights.push(100);
            }
            return degreesArrayHeights;
        }

        /**
         *加载线
         */
        function addPolyline(features, Symbol) {
            let point = [];
            for (let i = 0; i < features.geometry.coordinates.length; i++) {
                point.push(features.geometry.coordinates[i][0]);
                point.push(features.geometry.coordinates[i][1]);
            }
            linescollection = new Cesium.Primitive({
                geometryInstances: new Cesium.GeometryInstance({
                    geometry: new Cesium.PolylineGeometry({
                        positions: Cesium.Cartesian3.fromDegreesArray(point),
                        width: Symbol.Width,
                    }),
                    attributes: {
                        color: Cesium.ColorGeometryInstanceAttribute.fromColor(new Cesium.Color.fromCssColorString(Symbol.color))
                    }
                }),
                appearance: new Cesium.PolylineColorAppearance({
                    translucent: Symbol.translucent
                })
            });
            collection.add(linescollection);
        }

        /**
         *加载标签
         */
        function addLabel(features, position, Symbol) {
            labels.add({
                position: position,
                text: features.properties.Text,
                fillColor: new Cesium.Color.fromCssColorString(Symbol.fillColor),
                scale: Symbol.scale,
                font: Symbol.font,
                distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, Symbol.distanceDisplayCondition),
                verticalOrigin: Cesium.VerticalOrigin.BOTTOM,
                scaleByDistance: new Cesium.NearFarScalar(50000, 1, 500000, 0.5),
                style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                pixelOffset: new Cesium.Cartesian2(0, -10),
                outlineWidth: Symbol.outlineWidth,
                outlineColor: new Cesium.Color.fromCssColorString(Symbol.outlineColor)
            });

        }

        /**
         *加载发光线
         */
        function addLineLighting(features, Symbol) {
            let yellowMaterial = new xt3d.PolylineObject.PolylineLightingMaterialProperty(Cesium.Color.BLUE);
            let promise = Cesium.GeoJsonDataSource.load(features, {
                clampToGround: true
            });
            promise.then((dataSource) => {
                let values = dataSource.entities.values;
                for (let i = 0; i < values.length; i++) {
                    let line = values[i];
                    line.polyline.material = yellowMaterial;
                    line.polyline.width = Symbol.Width;
                    entitycollection.entities.add(new Cesium.Entity(values[i]));
                }
            }).otherwise(function (error) {
                console.log(error)
            });
        }

        /**
         *加载尾迹线
         */
        function wakeLine(features, Symbol) {
            let promise = Cesium.GeoJsonDataSource.load(features, {
                clampToGround: true
            });
            promise.then((dataSource) => {
                let values = dataSource.entities.values;
                for (let i = 0; i < values.length; i++) {
                    let line = values[i];
                    line.polyline.material = new xt3d.PolylineObject.PolylineTrailMaterialProperty({
                        speed: Symbol.speed * Math.random(),
                        color: new Cesium.Color.fromCssColorString(Symbol.color),
                        percent: Symbol.percent,
                        gradient: Symbol.gradient,
                    });
                    line.polyline.width = Symbol.Width;
                    entitycollection.entities.add(new Cesium.Entity(line));
                }
            }).otherwise(function (error) {
                console.log(error)
            });
        }


        // cesium.layer3DList[that.id] = that;
    }
    return OpenJsonService;
}