import * as Cesium from "cesium";
import mapRequest from "@/utils/map/mapRequest";
import { getHeightByType } from "@/utils/map/mapUtils";

export class RenderWithPrimitive {
    constructor(viewer, url, style) {
        this.features = null;
        this.viewer = viewer;
        this.url = url;
        this.style = style;
        this.visibleLevel = (this.style && this.style.level) ? this.style.level : 20000;
        this.positionHeight = (this.style && this.style.positionHeight) ? this.style.positionHeight : 15;
        this.imageUrl = (this.style && this.style.imageUrl) ? this.style.imageUrl : "";
        this.fontSize = (this.style && this.style.size) ? this.style.size : 25;
        this.fontColor = (this.style && this.style.color) ? new Cesium.Color.fromCssColorString(this.style.color).withAlpha(1) : Cesium.Color.RED;
        //Label方式
        // this.labels = this.viewer.scene.primitives.add(new Cesium.LabelCollection());
        //Entity方式
        this.labels = new Cesium.CustomDataSource("labels");
        this.viewer.dataSources.add(this.labels);
        // $.ajaxSettings.crossDomain = false;
        // $.ajaxSettings.xhrFields = {
        //     withCredentials: false
        // };
        // this.features = InitialParameter(this.url);
        let that = this;
        mapRequest({
            url: url,
            method: 'get',
        }).then((res) => {
            data = typeof res == "string" ? JSON.parse(res) : res

            that.features = data;
            // this.getHeight();
            this.addPostRender();
        })
    }

    getHeight() {
        var that = this;
        if (viewer.terrainProvider) {
            var allPoints = [];
            for (var p = 0; p < this.features.length; p++) {
                allPoints.push(Cesium.Cartographic.fromDegrees(this.features[p]["X"], this.features[p]["Y"]));
            }
            try {
                var promise = Cesium.sampleTerrain(viewer.terrainProvider, 15, allPoints);
                Cesium.when(promise, (updatedPositions) => {
                    for (var k = 0; k < updatedPositions.length; k++) {
                        that.features[k]["Z"] = updatedPositions[k].height;
                    }
                    that.addPostRender();
                    resolve(that.features)
                })
            } catch (e) {
                that.addPostRender();
            }
        } else {
            this.addPostRender();
        }
    }

    //添加场景事件
    addPostRender() {
        this.viewer.scene.postRender.addEventListener(this.postRender, this);
    }

    //场景渲染事件 实时更新标签的位置 使其与笛卡尔坐标一致
    postRender() {
        if (!this.features.length) {
            this.viewer.scene.postRender.removeEventListener(this.postRender, this); //移除事件监听
            return;
        }
        var that = this;
        var extent = getCurrentExtent({ height: this.visibleLevel });
        var featureList = [];
        if (extent) {
            featureList = this.ergodicFeatures(extent);
        }
        if (featureList.length > 0) {
            var allPoints = [];
            for (var p = 0; p < featureList.length; p++) {
                allPoints.push(Cesium.Cartographic.fromDegrees(featureList[p]["X"], featureList[p]["Y"]));
            }
            getHeightByType(allPoints, 'terrain').then(results => {
                for (var k = 0; k < results.length; k++) {
                    //Entity方式
                    that.labels.entities.add({
                        name: '',
                        position: new Cesium.Cartesian3.fromDegrees(featureList[k]["X"], featureList[k]["Y"], (results[k]["height"] || 0) + that.positionHeight),
                        label: { //文字标签
                            text: featureList[k]["NAME"],
                            scale: 0.6,
                            font: 'normal ' + that.fontSize + 'px MicroSoft YaHei',
                            scaleByDistance: new Cesium.NearFarScalar(that.visibleLevel / 10, 1, that.visibleLevel, 0.6),
                            style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                            fillColor: that.fontColor,
                            horizontalOrigin: Cesium.HorizontalOrigin.LEFT,
                            verticalOrigin: Cesium.VerticalOrigin.CENTER,
                            distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, that.visibleLevel),
                            //heightReference: Cesium.HeightReference.RELATIVE_TO_GROUND, //CLAMP_TO_GROUND//RELATIVE_TO_GROUND
                            outlineWidth: 3,
                            outlineColor: Cesium.Color.WHITE,
                            pixelOffset: new Cesium.Cartesian2(10, -2), //偏移量
                            //eyeOffset: new Cesium.Cartesian3(5, 0, 0)
                            //eyeOffset: new Cesium.Cartesian3(0, 0, -150)
                        },
                        billboard: {
                            image: that.imageUrl,
                            show: that.imageUrl ? true : false,
                            width: that.fontSize,
                            height: that.fontSize,
                            horizontalOrigin: Cesium.HorizontalOrigin.CENTER,
                            verticalOrigin: Cesium.VerticalOrigin.CENTER,
                            scale: 0.6,
                            distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, that.visibleLevel),
                        }
                    });
                    //Label方式
                    // that.labels.add({
                    //     position: new Cesium.Cartesian3.fromDegrees(featureList[k]["X"], featureList[k]["Y"], (results[k]["height"] || 0) + 20),
                    //     //text: "<div style='width: 10px;height: 10px;background: red;'></div>" + featureList[k]["NAME"],
                    //     text: featureList[k]["NAME"],
                    //     scale: 0.6,
                    //     font: 'normal ' + that.fontSize + 'px MicroSoft YaHei',
                    //     scaleByDistance: new Cesium.NearFarScalar(that.visibleLevel / 10, 1, that.visibleLevel, 0.6),
                    //     style: Cesium.LabelStyle.FILL_AND_OUTLINE,
                    //     fillColor: that.fontColor,
                    //     horizontalOrigin: Cesium.HorizontalOrigin.CENTER,
                    //     verticalOrigin: Cesium.VerticalOrigin.TOP,
                    //     distanceDisplayCondition: new Cesium.DistanceDisplayCondition(0, that.visibleLevel),
                    //     heightReference: Cesium.HeightReference.RELATIVE_TO_GROUND, //CLAMP_TO_GROUND//RELATIVE_TO_GROUND
                    //     outlineWidth: 2,
                    //     outlineColor: Cesium.Color.WHITE,
                    //     eyeOffset: new Cesium.Cartesian3(0, 0, -150)
                    // })
                }
            })
        }
    }

    //根据条件筛选需要显示的要素
    ergodicFeatures(extent) {
        var xmin = extent["xmin"];
        var xmax = extent["xmax"];
        var ymin = extent["ymin"];
        var ymax = extent["ymax"];
        var features = [];
        if (xmin && xmax && ymin && ymax) {
            for (var i = 0; i < this.features.length;) {
                var curX = this.features[i]["X"];
                var curY = this.features[i]["Y"];
                if (curX >= xmin && curX <= xmax && curY >= ymin && curY <= ymax) {
                    features.push(this.features[i]);
                    this.features.splice(i, 1);
                } else {
                    i++;
                }
            }
        } else { }
        return features;
    }

    setVisible(visible) {
        if (typeof (visible) == "undefined") {
            this.labels.show = !this.labels.show;
        } else {
            this.labels.show = visible;
        }
    }

    //关闭
    remove() {
        this.viewer.scene.postRender.removeEventListener(this.postRender, this); //移除事件监听
        this.labels = this.viewer.scene.primitives.remove(this.labels);
    }
}