﻿/**
* 本文件用于调用geoserver切割的瓦片
* @author Wilson.Yan
* @version 2016.8.21
*/
//改变数字格式
import * as Cesium from "cesium";
import { checkLayerTransparent } from "@/utils/map/layerHelper"

export function TileMongodbService(layerInfo) {
    var geoTiles = new Object;
    geoTiles.id = layerInfo.GUID;
    geoTiles.name = layerInfo.NAME;
    geoTiles.url = layerInfo.URL;
    geoTiles.wkid = layerInfo.hasOwnProperty("WKID") ? layerInfo.WKID : "3857";
    geoTiles.bbox = layerInfo.BBOX;
    geoTiles.miniresolution = layerInfo.hasOwnProperty("MINIRESOLUTION") ? layerInfo.MINIRESOLUTION : 0;
    geoTiles.maxresolution = layerInfo.hasOwnProperty("MAXRESOLUTION") ? layerInfo.MAXRESOLUTION : 19;
    geoTiles.minzoom = layerInfo.hasOwnProperty("MINZOOM") ? layerInfo.MINZOOM : 0;
    geoTiles.maxzoom = layerInfo.hasOwnProperty("MAXZOOM") ? layerInfo.MAXZOOM : 20;
    geoTiles.prj = layerInfo.PRJ;
    geoTiles.CreateLayer = function (viewer) {
        var id = this.id;
        //内蒙行政区范围过滤筛选
        var mapExtent = {
            "WuHai": {
                "minx": 106.605639,
                "miny": 39.036193,
                "maxx": 107.134902,
                "maxy": 39.915614
            },
            "EErDuoSi": {
                "minx": 106.478327,
                "miny": 37.620923,
                "maxx": 111.456502,
                "maxy": 40.880766
            },
            "HuShi": {
                "minx": 110.511417,
                "miny": 39.586325,
                "maxx": 112.305153,
                "maxy": 41.385562
            },
            "BaYanNaoEr": {
                "minx": 105.195084,
                "miny": 40.167523,
                "maxx": 109.881557,
                "maxy": 42.463299
            },
            "BaoTou": {
                "minx": 109.254218,
                "miny": 40.246398,
                "maxx": 111.439588,
                "maxy": 42.734829
            },
            "ALaShan": {
                "minx": 97.171724,
                "miny": 37.406642,
                "maxx": 106.862027,
                "maxy": 42.794526
            },
            "WuLanChaBu": {
                "minx": 110.332830,
                "miny": 40.162147,
                "maxx": 114.802754,
                "maxy": 43.375592
            },
            "ChiFeng": {
                "minx": 116.349864,
                "miny": 41.285246,
                "maxx": 120.980773,
                "maxy": 45.250261
            },
            "TongLiao": {
                "minx": 119.241264,
                "miny": 42.240271,
                "maxx": 123.714890,
                "maxy": 45.654075
            },
            "XiLinGuoLe": {
                "minx": 111.142000,
                "miny": 41.569306,
                "maxx": 119.978042,
                "maxy": 46.775041
            },
            "XingAnMeng": {
                "minx": 119.473660,
                "miny": 44.253266,
                "maxx": 123.625278,
                "maxy": 47.654892
            },
            "HuLunBeiEr": {
                "minx": 115.521412,
                "miny": 47.093644,
                "maxx": 126.068735,
                "maxy": 53.333741
            }
        };
        var rectangle;
        if (this.url.indexOf("DOM_07_") > -1) {
            let current = this.url.split("DOM_07_")[1].split("/")[0];
            if (mapExtent[current]) {
                rectangle = Cesium.Rectangle.fromDegrees(mapExtent[current]["minx"], mapExtent[current]["miny"], mapExtent[current]["maxx"], mapExtent[current]["maxy"]);
            }
        }
        var thisLayer;
        var layers = viewer.imageryLayers;
        var layerindex = layers._layers.length;
        if (rectangle) {
            thisLayer = layers.addImageryProvider(new Cesium.UrlTemplateImageryProvider({
                url: this.url, //服务地址
                maximumLevel: this.maxzoom,
                minimumLevel: this.minzoom,
                rectangle: rectangle
            }));
        } else {
            thisLayer = layers.addImageryProvider(new Cesium.UrlTemplateImageryProvider({
                url: this.url, //服务地址
                maximumLevel: this.maxzoom,
                minimumLevel: this.minzoom
            }));
        }
        return thisLayer;
    };
    return geoTiles;
}