import myMap from "@/utils/arcMap.js";
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import {
  Fill,
  Stroke,
  Circle,
  Style
} from 'ol/style';
import Draw from 'ol/interaction/Draw';
import DoubleClickZoom from 'ol/interaction/DoubleClickZoom';
import Polygon from 'ol/geom/Polygon';
import LineString from 'ol/geom/LineString';
import {
  getLength,
  getArea
} from 'ol/sphere';
import Overlay from 'ol/Overlay';
import {
  unByKey
} from 'ol/Observable';
import Icon from 'ol/style/Icon';
import Feature from 'ol/Feature';
import {
  fromLonLat
} from 'ol/proj';
import Point from 'ol/geom/Point';
//地图工具
var OLTool = function () {
  var Map = myMap.map;
  var that = this;
  this.Draws = function (op) {
    op = op || {};
    var _style_ = op.style ||
      new Style({
        fill: new Fill({
          color: 'rgba(255, 255, 255, 0.2)'
        }),
        stroke: new Stroke({
          color: '#ffcc33',
          width: 2
        }),
        image: new Circle({
          radius: 7,
          fill: new Fill({
            color: '#ffcc33'
          })
        })
      });
    // var _layer = myMap.CreateVecLayer("draw_Layer", _style_)
    // myMap.addLayer(_layer);
    var _layer = myMap.tempLayers.dview_draw_Layer;
    if (op.layer) {
      _layer = myMap.getLayer(op.layer);
      if (_layer == null) {
        _layer = new VectorLayer({
          name: op.layer,
          source: VectorSource({
            wrapX: false
          })
        });
        myMap.addLayer(_layer);
      }
    }
    _layer.setVisible(true);
    var _draw = {};
    _draw._layer = _layer;
    _draw._state = null;
    if(op.type){
      _draw._type = op.type;
      _draw.interaction = new Draw({
        source: _layer.getSource(),
        type: type,
        style: _style_,
      });
    }
    _draw.Start = function () {
      _draw._state = "open";
      document.getElementById(myMap._domid).setAttribute('style', 'cursor: crosshair !important');
      Map.getInteractions().forEach(function (interaction, index) {
        if (interaction instanceof DoubleClickZoom)
          interaction.setActive(false);
      });
      _draw.SetType(_draw._type)
    };
    _draw.End = function () {
      document.getElementById(myMap._domid).setAttribute('style', 'cursor:"" !important');
      _draw._state = "close";
      console.log(_draw)
      Map.removeInteraction(this.interaction);
    };
    _draw.GetState = function () {
      return _draw._state;
    }
    _draw.SetType = function (type, geometryFunction) {
      _draw._type = type;
      Map.removeInteraction(this.interaction);
      var _i = this.interaction = new Draw({
        source: _layer.getSource(),
        type: _draw._type,
        style: _style_,
      });
      _action.forEach(function () {
        _i.on(this.event, this.fn);
      })
      Map.addInteraction(this.interaction);
    };
    var _action = [];
    _draw.on = function (event, fn) {
      _action.push({
        event: event,
        fn: fn
      });
      _draw.interaction.on(event, fn);
    };
    _draw.un = function (event) {
      _action.forEach(function (i, obj) {
        if (this.event == event) {
          _action.splice(i, 1);
          _draw.interaction.un(event, obj[event])
        }
      })
    }
    _draw.ClearAll = function () {
      _layer.getSource().clear();
    };
    return _draw;
  };



  this.Meatures = function (type, option) {
    var _option = option || {
      continuePolygonMsg: '点击开始测量面积',
      continueLineMsg: '点击开始测量长度',
      helpMsg: "开始测量"
    }
    var sketch;
    var helpTooltipElement;
    var helpTooltip;
    var measureTooltipElement;
    var measureTooltip;

    var configDic = {
      "Area": "Polygon",
      "Length": "LineString"
    };
    var _meature = {};
    var pointerMoveHandler = function (evt) {
      if (evt.dragging) {
        return;
      }
      /** @type {string} */
      var helpMsg = _option.helpMsg;
      if (sketch) {
        var geom = (sketch.getGeometry());
        if (geom instanceof Polygon) {
          helpMsg = _option.continuePolygonMsg;
        } else if (geom instanceof LineString) {
          helpMsg = _option.continueLineMsg;
        }
      }
      helpTooltipElement.innerHTML = helpMsg;
      helpTooltip.setPosition(evt.coordinate);

      helpTooltipElement.classList.remove('hidden');
    };
    /*格式化长度*/
    // var formatLength = function (line) {
    //   var length = getLength(line);
    //   var output;
    //   if (length > 100) {
    //     output = (Math.round(length / 1000 * 100) / 100) +
    //       ' ' + 'km';
    //   } else {
    //     output = (Math.round(length * 100) / 100) +
    //       ' ' + 'm';
    //   }
    //   return output;
    // };
    /*格式化面积*/
    // var formatArea = function (polygon) {
    //   var area = getArea(polygon);
    //   var output;
    //   if (area > 10000) {
    //     output = (Math.round(area / 1000000 * 100) / 100) +
    //       ' ' + 'km<sup>2</sup>';
    //   } else {
    //     output = (Math.round(area * 100) / 100) +
    //       ' ' + 'm<sup>2</sup>';
    //   }
    //   return output;
    // };
    /*创建帮助提示*/
    function createHelpTooltip() {
      if (helpTooltipElement) {
        helpTooltipElement.parentNode.removeChild(helpTooltipElement);
      }
      helpTooltipElement = document.createElement('div');
      helpTooltipElement.className = 'tooltip hidden';
      helpTooltip = new Overlay({
        element: helpTooltipElement,
        offset: [15, 0],
        positioning: 'center-left'
      });
      myMap.map.addOverlay(helpTooltip);
    }
    /*创建测量的帮助提示*/
    function createMeasureTooltip() {
      if (measureTooltipElement) {
        measureTooltipElement.parentNode.removeChild(measureTooltipElement);
      }
      measureTooltipElement = document.createElement('div');
      measureTooltipElement.className = 'tooltip tooltip-measure';
      measureTooltip = new Overlay({
        element: measureTooltipElement,
        offset: [0, -15],
        positioning: 'bottom-center'
      });
      myMap.map.addOverlay(measureTooltip);
    }
    var meature_Draws = new this.Draws();
    _meature._draw = meature_Draws;
    _meature.Start = function () {
      createMeasureTooltip();
      createHelpTooltip();
      meature_Draws.Start();
      var listener;
      // document.getElementById(myMap._domid).setAttribute('style', 'cursor: url("~@/assets/images/ruler.cur"),crosshair !important');
      meature_Draws.on('drawstart', function (evt) {
        sketch = evt.feature;
        /** @type {ol.Coordinate|undefined} */
        var tooltipCoord = evt.coordinate;
        listener = sketch.getGeometry().on('change', function (evt) {
          var geom = evt.target;
          var output;
          if (geom instanceof Polygon) {
            output = that.Tools.formatArea(geom);
            tooltipCoord = geom.getInteriorPoint().getCoordinates();
          } else if (geom instanceof LineString) {
            output = that.Tools.formatLength(geom);
            tooltipCoord = geom.getLastCoordinate();
          }
          measureTooltipElement.innerHTML = output;
          measureTooltip.setPosition(tooltipCoord);
        });
      }, this);
      meature_Draws.on('drawend', function () {
        measureTooltipElement.className = 'tooltip tooltip-static';
        measureTooltip.setOffset([0, -7]);
        sketch = null;
        measureTooltipElement = null;
        createMeasureTooltip();
        unByKey(listener);
        _meature.End();
      }, this);
    }
    _meature.End = function () {
      document.getElementById(myMap._domid).setAttribute('style', 'cursor:  !important');
      meature_Draws.End();
    }
    _meature.SetType = function (type) {
      if (type == "Area") {
        meature_Draws.SetType("Polygon");
      } else if (type == "Length") {
        meature_Draws.SetType("LineString");
      }
    };

    return _meature;
  };

  //定位到点
  this.Common = function (x, y) {
    this.CommonLayer = myMap.getLayer("common_layer");
    if(this.CommonLayer == null){
      this.CommonLayer = new VectorLayer({
        source: new VectorSource({
          features: []
        }),
        name: "common_layer",
        zIndex: 999999999999
      });
      myMap.addLayer(this.CommonLayer);
    }

    this.QueryLocation = function (x, y, style) {
      var _hideStyle = new Style({
        image: new Icon( /** @type {olx.style.IconOptions} */ ({
          color: [0, 109, 255],
          opacity: 0,
          anchor: [0.5, 1],
          scale: 0.7,
          crossOrigin: 'anonymous',
          src: '/images/marker_red.png'
        }))
      });
      var myicon = style || new Style({
        image: new Icon( /** @type {olx.style.IconOptions} */ ({
          color: [0, 109, 255, 0.5],
          anchor: [0.5, 1],
          scale: 0.7,
          crossOrigin: 'anonymous',
          src: '/images/marker_red.png'
        }))
      });
      var pt;
      if (myMap._EPSG == myMap.EPSG.C3857) {
        pt = new Feature({
          geometry: new Point(fromLonLat([parseFloat(x), parseFloat(y)]))
        });
        //MyMap.zoomToPoint(x, y, true);
        myMap.map.getView().setCenter(fromLonLat([parseFloat(x), parseFloat(y)]))
      } else {
        pt = new Feature({
          geometry: new Point([parseFloat(x), parseFloat(y)])
        });
        //MyMap.zoomToPoint(x, y, false);
        myMap.map.getView().setCenter(fromLonLat([parseFloat(x), parseFloat(y)]))
      }
      pt.setStyle(myicon);
      this.CommonLayer.getSource().addFeature(pt);
    }
  };

  this.Tools = {
    formatLength: function (line, m) {
      var length = getLength(line);
      var output;
      if(m){
        output = (Math.round(area * 100) / 100);
      }else{
        if (length > 100) {
          output = (Math.round(length / 1000 * 100) / 100) +
            ' ' + 'km';
        } else {
          output = (Math.round(length * 100) / 100) +
            ' ' + 'm';
        }
      }
      return output;
    },
    formatArea: function (polygon, m) {
      var area = getArea(polygon);
      var output;
      if(m){
        output = (Math.round(area * 100) / 100);
      }else{
        if (area > 10000) {
          output = (Math.round(area / 1000000 * 100) / 100) +
            ' ' + 'km²';
        } else {
          output = (Math.round(area * 100) / 100) +
            ' ' + 'm²';
        }
      }
      return output;
    }
  }
}
export default OLTool;